from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="UploadDocumentsResponse")



@_attrs_define
class UploadDocumentsResponse:
    """ Response for document upload operations.

    Returns array of document IDs in the same order as uploaded files.
    Includes duplicate warnings for files with identical content already in workspace.

        Attributes:
            doc_ext_ids (list[str]):
            duplicates (list[str] | Unset): Doc IDs that have identical content to existing documents in the workspace
     """

    doc_ext_ids: list[str]
    duplicates: list[str] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        doc_ext_ids = self.doc_ext_ids



        duplicates: list[str] | Unset = UNSET
        if not isinstance(self.duplicates, Unset):
            duplicates = self.duplicates




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "doc_ext_ids": doc_ext_ids,
        })
        if duplicates is not UNSET:
            field_dict["duplicates"] = duplicates

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        doc_ext_ids = cast(list[str], d.pop("doc_ext_ids"))


        duplicates = cast(list[str], d.pop("duplicates", UNSET))


        upload_documents_response = cls(
            doc_ext_ids=doc_ext_ids,
            duplicates=duplicates,
        )


        upload_documents_response.additional_properties = d
        return upload_documents_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast

if TYPE_CHECKING:
  from ..models.doc_metadata import DocMetadata





T = TypeVar("T", bound="DocUpdateRequest")



@_attrs_define
class DocUpdateRequest:
    """ Document update request - identifies doc and fields to update.

        Attributes:
            external_id (str):
            shared (bool | None | Unset):
            doc_metadata (DocMetadata | None | Unset):
            status (Literal['processing'] | None | Unset):
            content (None | str | Unset):
            folder (None | str | Unset): Logical folder path (URL safe)
     """

    external_id: str
    shared: bool | None | Unset = UNSET
    doc_metadata: DocMetadata | None | Unset = UNSET
    status: Literal['processing'] | None | Unset = UNSET
    content: None | str | Unset = UNSET
    folder: None | str | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.doc_metadata import DocMetadata
        external_id = self.external_id

        shared: bool | None | Unset
        if isinstance(self.shared, Unset):
            shared = UNSET
        else:
            shared = self.shared

        doc_metadata: dict[str, Any] | None | Unset
        if isinstance(self.doc_metadata, Unset):
            doc_metadata = UNSET
        elif isinstance(self.doc_metadata, DocMetadata):
            doc_metadata = self.doc_metadata.to_dict()
        else:
            doc_metadata = self.doc_metadata

        status: Literal['processing'] | None | Unset
        if isinstance(self.status, Unset):
            status = UNSET
        else:
            status = self.status

        content: None | str | Unset
        if isinstance(self.content, Unset):
            content = UNSET
        else:
            content = self.content

        folder: None | str | Unset
        if isinstance(self.folder, Unset):
            folder = UNSET
        else:
            folder = self.folder


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "external_id": external_id,
        })
        if shared is not UNSET:
            field_dict["shared"] = shared
        if doc_metadata is not UNSET:
            field_dict["doc_metadata"] = doc_metadata
        if status is not UNSET:
            field_dict["status"] = status
        if content is not UNSET:
            field_dict["content"] = content
        if folder is not UNSET:
            field_dict["folder"] = folder

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.doc_metadata import DocMetadata
        d = dict(src_dict)
        external_id = d.pop("external_id")

        def _parse_shared(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        shared = _parse_shared(d.pop("shared", UNSET))


        def _parse_doc_metadata(data: object) -> DocMetadata | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                doc_metadata_type_0 = DocMetadata.from_dict(data)



                return doc_metadata_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DocMetadata | None | Unset, data)

        doc_metadata = _parse_doc_metadata(d.pop("doc_metadata", UNSET))


        def _parse_status(data: object) -> Literal['processing'] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            status_type_0 = cast(Literal['processing'] , data)
            if status_type_0 != 'processing':
                raise ValueError(f"status_type_0 must match const 'processing', got '{status_type_0}'")
            return status_type_0
            return cast(Literal['processing'] | None | Unset, data)

        status = _parse_status(d.pop("status", UNSET))


        def _parse_content(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        content = _parse_content(d.pop("content", UNSET))


        def _parse_folder(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        folder = _parse_folder(d.pop("folder", UNSET))


        doc_update_request = cls(
            external_id=external_id,
            shared=shared,
            doc_metadata=doc_metadata,
            status=status,
            content=content,
            folder=folder,
        )

        return doc_update_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.agents_config import AgentsConfig
  from ..models.chunker_config import ChunkerConfig
  from ..models.doctag_llm_config import DoctagLLMConfig
  from ..models.embedder_config import EmbedderConfig
  from ..models.evaluator_llm_config import EvaluatorLLMConfig
  from ..models.keyword_embedder_config import KeywordEmbedderConfig
  from ..models.memory_llm_config import MemoryLLMConfig
  from ..models.model_citation_config import ModelCitationConfig
  from ..models.parser_config import ParserConfig
  from ..models.planning_llm_config import PlanningLLMConfig
  from ..models.query_llm_config import QueryLLMConfig
  from ..models.reranker_config import RerankerConfig
  from ..models.retriever_config import RetrieverConfig
  from ..models.review_llm_config import ReviewLLMConfig
  from ..models.summarise_llm_config import SummariseLLMConfig
  from ..models.title_llm_config import TitleLLMConfig
  from ..models.vision_llm_config import VisionLLMConfig





T = TypeVar("T", bound="ConfigUpdateData")



@_attrs_define
class ConfigUpdateData:
    """ Configuration update - only changed parameters need to be provided with proper validation

        Attributes:
            query_llm (None | QueryLLMConfig | Unset):
            review_llm (None | ReviewLLMConfig | Unset):
            agents (AgentsConfig | None | Unset):
            evaluator_llm (EvaluatorLLMConfig | None | Unset):
            title_llm (None | TitleLLMConfig | Unset):
            summarise_llm (None | SummariseLLMConfig | Unset):
            doctag_llm (DoctagLLMConfig | None | Unset):
            memory_llm (MemoryLLMConfig | None | Unset):
            planning_llm (None | PlanningLLMConfig | Unset):
            vision_llm (None | Unset | VisionLLMConfig):
            model_citation (ModelCitationConfig | None | Unset):
            parser (None | ParserConfig | Unset):
            chunker (ChunkerConfig | None | Unset):
            embedder (EmbedderConfig | None | Unset):
            retriever (None | RetrieverConfig | Unset):
            reranker (None | RerankerConfig | Unset):
            keyword_embedder (KeywordEmbedderConfig | None | Unset):
            title (str | Unset):  Default: ''.
            parent_message_ext_id (None | str | Unset):
     """

    query_llm: None | QueryLLMConfig | Unset = UNSET
    review_llm: None | ReviewLLMConfig | Unset = UNSET
    agents: AgentsConfig | None | Unset = UNSET
    evaluator_llm: EvaluatorLLMConfig | None | Unset = UNSET
    title_llm: None | TitleLLMConfig | Unset = UNSET
    summarise_llm: None | SummariseLLMConfig | Unset = UNSET
    doctag_llm: DoctagLLMConfig | None | Unset = UNSET
    memory_llm: MemoryLLMConfig | None | Unset = UNSET
    planning_llm: None | PlanningLLMConfig | Unset = UNSET
    vision_llm: None | Unset | VisionLLMConfig = UNSET
    model_citation: ModelCitationConfig | None | Unset = UNSET
    parser: None | ParserConfig | Unset = UNSET
    chunker: ChunkerConfig | None | Unset = UNSET
    embedder: EmbedderConfig | None | Unset = UNSET
    retriever: None | RetrieverConfig | Unset = UNSET
    reranker: None | RerankerConfig | Unset = UNSET
    keyword_embedder: KeywordEmbedderConfig | None | Unset = UNSET
    title: str | Unset = ''
    parent_message_ext_id: None | str | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        from ..models.doctag_llm_config import DoctagLLMConfig
        from ..models.title_llm_config import TitleLLMConfig
        from ..models.retriever_config import RetrieverConfig
        from ..models.reranker_config import RerankerConfig
        from ..models.model_citation_config import ModelCitationConfig
        from ..models.keyword_embedder_config import KeywordEmbedderConfig
        from ..models.evaluator_llm_config import EvaluatorLLMConfig
        from ..models.review_llm_config import ReviewLLMConfig
        from ..models.query_llm_config import QueryLLMConfig
        from ..models.agents_config import AgentsConfig
        from ..models.memory_llm_config import MemoryLLMConfig
        from ..models.planning_llm_config import PlanningLLMConfig
        from ..models.summarise_llm_config import SummariseLLMConfig
        from ..models.embedder_config import EmbedderConfig
        from ..models.chunker_config import ChunkerConfig
        from ..models.parser_config import ParserConfig
        from ..models.vision_llm_config import VisionLLMConfig
        query_llm: dict[str, Any] | None | Unset
        if isinstance(self.query_llm, Unset):
            query_llm = UNSET
        elif isinstance(self.query_llm, QueryLLMConfig):
            query_llm = self.query_llm.to_dict()
        else:
            query_llm = self.query_llm

        review_llm: dict[str, Any] | None | Unset
        if isinstance(self.review_llm, Unset):
            review_llm = UNSET
        elif isinstance(self.review_llm, ReviewLLMConfig):
            review_llm = self.review_llm.to_dict()
        else:
            review_llm = self.review_llm

        agents: dict[str, Any] | None | Unset
        if isinstance(self.agents, Unset):
            agents = UNSET
        elif isinstance(self.agents, AgentsConfig):
            agents = self.agents.to_dict()
        else:
            agents = self.agents

        evaluator_llm: dict[str, Any] | None | Unset
        if isinstance(self.evaluator_llm, Unset):
            evaluator_llm = UNSET
        elif isinstance(self.evaluator_llm, EvaluatorLLMConfig):
            evaluator_llm = self.evaluator_llm.to_dict()
        else:
            evaluator_llm = self.evaluator_llm

        title_llm: dict[str, Any] | None | Unset
        if isinstance(self.title_llm, Unset):
            title_llm = UNSET
        elif isinstance(self.title_llm, TitleLLMConfig):
            title_llm = self.title_llm.to_dict()
        else:
            title_llm = self.title_llm

        summarise_llm: dict[str, Any] | None | Unset
        if isinstance(self.summarise_llm, Unset):
            summarise_llm = UNSET
        elif isinstance(self.summarise_llm, SummariseLLMConfig):
            summarise_llm = self.summarise_llm.to_dict()
        else:
            summarise_llm = self.summarise_llm

        doctag_llm: dict[str, Any] | None | Unset
        if isinstance(self.doctag_llm, Unset):
            doctag_llm = UNSET
        elif isinstance(self.doctag_llm, DoctagLLMConfig):
            doctag_llm = self.doctag_llm.to_dict()
        else:
            doctag_llm = self.doctag_llm

        memory_llm: dict[str, Any] | None | Unset
        if isinstance(self.memory_llm, Unset):
            memory_llm = UNSET
        elif isinstance(self.memory_llm, MemoryLLMConfig):
            memory_llm = self.memory_llm.to_dict()
        else:
            memory_llm = self.memory_llm

        planning_llm: dict[str, Any] | None | Unset
        if isinstance(self.planning_llm, Unset):
            planning_llm = UNSET
        elif isinstance(self.planning_llm, PlanningLLMConfig):
            planning_llm = self.planning_llm.to_dict()
        else:
            planning_llm = self.planning_llm

        vision_llm: dict[str, Any] | None | Unset
        if isinstance(self.vision_llm, Unset):
            vision_llm = UNSET
        elif isinstance(self.vision_llm, VisionLLMConfig):
            vision_llm = self.vision_llm.to_dict()
        else:
            vision_llm = self.vision_llm

        model_citation: dict[str, Any] | None | Unset
        if isinstance(self.model_citation, Unset):
            model_citation = UNSET
        elif isinstance(self.model_citation, ModelCitationConfig):
            model_citation = self.model_citation.to_dict()
        else:
            model_citation = self.model_citation

        parser: dict[str, Any] | None | Unset
        if isinstance(self.parser, Unset):
            parser = UNSET
        elif isinstance(self.parser, ParserConfig):
            parser = self.parser.to_dict()
        else:
            parser = self.parser

        chunker: dict[str, Any] | None | Unset
        if isinstance(self.chunker, Unset):
            chunker = UNSET
        elif isinstance(self.chunker, ChunkerConfig):
            chunker = self.chunker.to_dict()
        else:
            chunker = self.chunker

        embedder: dict[str, Any] | None | Unset
        if isinstance(self.embedder, Unset):
            embedder = UNSET
        elif isinstance(self.embedder, EmbedderConfig):
            embedder = self.embedder.to_dict()
        else:
            embedder = self.embedder

        retriever: dict[str, Any] | None | Unset
        if isinstance(self.retriever, Unset):
            retriever = UNSET
        elif isinstance(self.retriever, RetrieverConfig):
            retriever = self.retriever.to_dict()
        else:
            retriever = self.retriever

        reranker: dict[str, Any] | None | Unset
        if isinstance(self.reranker, Unset):
            reranker = UNSET
        elif isinstance(self.reranker, RerankerConfig):
            reranker = self.reranker.to_dict()
        else:
            reranker = self.reranker

        keyword_embedder: dict[str, Any] | None | Unset
        if isinstance(self.keyword_embedder, Unset):
            keyword_embedder = UNSET
        elif isinstance(self.keyword_embedder, KeywordEmbedderConfig):
            keyword_embedder = self.keyword_embedder.to_dict()
        else:
            keyword_embedder = self.keyword_embedder

        title = self.title

        parent_message_ext_id: None | str | Unset
        if isinstance(self.parent_message_ext_id, Unset):
            parent_message_ext_id = UNSET
        else:
            parent_message_ext_id = self.parent_message_ext_id


        field_dict: dict[str, Any] = {}

        field_dict.update({
        })
        if query_llm is not UNSET:
            field_dict["QueryLLM"] = query_llm
        if review_llm is not UNSET:
            field_dict["ReviewLLM"] = review_llm
        if agents is not UNSET:
            field_dict["Agents"] = agents
        if evaluator_llm is not UNSET:
            field_dict["EvaluatorLLM"] = evaluator_llm
        if title_llm is not UNSET:
            field_dict["TitleLLM"] = title_llm
        if summarise_llm is not UNSET:
            field_dict["SummariseLLM"] = summarise_llm
        if doctag_llm is not UNSET:
            field_dict["DoctagLLM"] = doctag_llm
        if memory_llm is not UNSET:
            field_dict["MemoryLLM"] = memory_llm
        if planning_llm is not UNSET:
            field_dict["PlanningLLM"] = planning_llm
        if vision_llm is not UNSET:
            field_dict["VisionLLM"] = vision_llm
        if model_citation is not UNSET:
            field_dict["ModelCitation"] = model_citation
        if parser is not UNSET:
            field_dict["Parser"] = parser
        if chunker is not UNSET:
            field_dict["Chunker"] = chunker
        if embedder is not UNSET:
            field_dict["Embedder"] = embedder
        if retriever is not UNSET:
            field_dict["Retriever"] = retriever
        if reranker is not UNSET:
            field_dict["Reranker"] = reranker
        if keyword_embedder is not UNSET:
            field_dict["KeywordEmbedder"] = keyword_embedder
        if title is not UNSET:
            field_dict["title"] = title
        if parent_message_ext_id is not UNSET:
            field_dict["parent_message_ext_id"] = parent_message_ext_id

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.agents_config import AgentsConfig
        from ..models.chunker_config import ChunkerConfig
        from ..models.doctag_llm_config import DoctagLLMConfig
        from ..models.embedder_config import EmbedderConfig
        from ..models.evaluator_llm_config import EvaluatorLLMConfig
        from ..models.keyword_embedder_config import KeywordEmbedderConfig
        from ..models.memory_llm_config import MemoryLLMConfig
        from ..models.model_citation_config import ModelCitationConfig
        from ..models.parser_config import ParserConfig
        from ..models.planning_llm_config import PlanningLLMConfig
        from ..models.query_llm_config import QueryLLMConfig
        from ..models.reranker_config import RerankerConfig
        from ..models.retriever_config import RetrieverConfig
        from ..models.review_llm_config import ReviewLLMConfig
        from ..models.summarise_llm_config import SummariseLLMConfig
        from ..models.title_llm_config import TitleLLMConfig
        from ..models.vision_llm_config import VisionLLMConfig
        d = dict(src_dict)
        def _parse_query_llm(data: object) -> None | QueryLLMConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                query_llm_type_0 = QueryLLMConfig.from_dict(data)



                return query_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | QueryLLMConfig | Unset, data)

        query_llm = _parse_query_llm(d.pop("QueryLLM", UNSET))


        def _parse_review_llm(data: object) -> None | ReviewLLMConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                review_llm_type_0 = ReviewLLMConfig.from_dict(data)



                return review_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ReviewLLMConfig | Unset, data)

        review_llm = _parse_review_llm(d.pop("ReviewLLM", UNSET))


        def _parse_agents(data: object) -> AgentsConfig | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                agents_type_0 = AgentsConfig.from_dict(data)



                return agents_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(AgentsConfig | None | Unset, data)

        agents = _parse_agents(d.pop("Agents", UNSET))


        def _parse_evaluator_llm(data: object) -> EvaluatorLLMConfig | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                evaluator_llm_type_0 = EvaluatorLLMConfig.from_dict(data)



                return evaluator_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(EvaluatorLLMConfig | None | Unset, data)

        evaluator_llm = _parse_evaluator_llm(d.pop("EvaluatorLLM", UNSET))


        def _parse_title_llm(data: object) -> None | TitleLLMConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                title_llm_type_0 = TitleLLMConfig.from_dict(data)



                return title_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | TitleLLMConfig | Unset, data)

        title_llm = _parse_title_llm(d.pop("TitleLLM", UNSET))


        def _parse_summarise_llm(data: object) -> None | SummariseLLMConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                summarise_llm_type_0 = SummariseLLMConfig.from_dict(data)



                return summarise_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | SummariseLLMConfig | Unset, data)

        summarise_llm = _parse_summarise_llm(d.pop("SummariseLLM", UNSET))


        def _parse_doctag_llm(data: object) -> DoctagLLMConfig | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                doctag_llm_type_0 = DoctagLLMConfig.from_dict(data)



                return doctag_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DoctagLLMConfig | None | Unset, data)

        doctag_llm = _parse_doctag_llm(d.pop("DoctagLLM", UNSET))


        def _parse_memory_llm(data: object) -> MemoryLLMConfig | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                memory_llm_type_0 = MemoryLLMConfig.from_dict(data)



                return memory_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(MemoryLLMConfig | None | Unset, data)

        memory_llm = _parse_memory_llm(d.pop("MemoryLLM", UNSET))


        def _parse_planning_llm(data: object) -> None | PlanningLLMConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                planning_llm_type_0 = PlanningLLMConfig.from_dict(data)



                return planning_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | PlanningLLMConfig | Unset, data)

        planning_llm = _parse_planning_llm(d.pop("PlanningLLM", UNSET))


        def _parse_vision_llm(data: object) -> None | Unset | VisionLLMConfig:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                vision_llm_type_0 = VisionLLMConfig.from_dict(data)



                return vision_llm_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | VisionLLMConfig, data)

        vision_llm = _parse_vision_llm(d.pop("VisionLLM", UNSET))


        def _parse_model_citation(data: object) -> ModelCitationConfig | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                model_citation_type_0 = ModelCitationConfig.from_dict(data)



                return model_citation_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ModelCitationConfig | None | Unset, data)

        model_citation = _parse_model_citation(d.pop("ModelCitation", UNSET))


        def _parse_parser(data: object) -> None | ParserConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                parser_type_0 = ParserConfig.from_dict(data)



                return parser_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ParserConfig | Unset, data)

        parser = _parse_parser(d.pop("Parser", UNSET))


        def _parse_chunker(data: object) -> ChunkerConfig | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                chunker_type_0 = ChunkerConfig.from_dict(data)



                return chunker_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ChunkerConfig | None | Unset, data)

        chunker = _parse_chunker(d.pop("Chunker", UNSET))


        def _parse_embedder(data: object) -> EmbedderConfig | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                embedder_type_0 = EmbedderConfig.from_dict(data)



                return embedder_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(EmbedderConfig | None | Unset, data)

        embedder = _parse_embedder(d.pop("Embedder", UNSET))


        def _parse_retriever(data: object) -> None | RetrieverConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                retriever_type_0 = RetrieverConfig.from_dict(data)



                return retriever_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RetrieverConfig | Unset, data)

        retriever = _parse_retriever(d.pop("Retriever", UNSET))


        def _parse_reranker(data: object) -> None | RerankerConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                reranker_type_0 = RerankerConfig.from_dict(data)



                return reranker_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RerankerConfig | Unset, data)

        reranker = _parse_reranker(d.pop("Reranker", UNSET))


        def _parse_keyword_embedder(data: object) -> KeywordEmbedderConfig | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                keyword_embedder_type_0 = KeywordEmbedderConfig.from_dict(data)



                return keyword_embedder_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(KeywordEmbedderConfig | None | Unset, data)

        keyword_embedder = _parse_keyword_embedder(d.pop("KeywordEmbedder", UNSET))


        title = d.pop("title", UNSET)

        def _parse_parent_message_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        parent_message_ext_id = _parse_parent_message_ext_id(d.pop("parent_message_ext_id", UNSET))


        config_update_data = cls(
            query_llm=query_llm,
            review_llm=review_llm,
            agents=agents,
            evaluator_llm=evaluator_llm,
            title_llm=title_llm,
            summarise_llm=summarise_llm,
            doctag_llm=doctag_llm,
            memory_llm=memory_llm,
            planning_llm=planning_llm,
            vision_llm=vision_llm,
            model_citation=model_citation,
            parser=parser,
            chunker=chunker,
            embedder=embedder,
            retriever=retriever,
            reranker=reranker,
            keyword_embedder=keyword_embedder,
            title=title,
            parent_message_ext_id=parent_message_ext_id,
        )

        return config_update_data




""" Contains all the data models used in inputs/outputs """

from .add_contacts_request import AddContactsRequest
from .add_workspace_users_request import AddWorkspaceUsersRequest
from .agent_step_event import AgentStepEvent
from .agent_step_event_detail_type_0_item import AgentStepEventDetailType0Item
from .agents_config import AgentsConfig
from .agents_config_agent_api_type import AgentsConfigAgentApiType
from .all_configs import AllConfigs
from .all_llm_calls_aggregate import AllLlmCallsAggregate
from .artifact_event import ArtifactEvent
from .artifact_event_mode import ArtifactEventMode
from .ask_user_detail import AskUserDetail
from .auth_message import AuthMessage
from .auth_result_message import AuthResultMessage
from .batch_complete_message import BatchCompleteMessage
from .batch_complete_message_batch_type import BatchCompleteMessageBatchType
from .change_password_request import ChangePasswordRequest
from .change_password_request_rewrapped_agent_recovery_keys_type_0 import ChangePasswordRequestRewrappedAgentRecoveryKeysType0
from .change_password_request_rewrapped_workspace_keys import ChangePasswordRequestRewrappedWorkspaceKeys
from .change_password_response import ChangePasswordResponse
from .chat_completion_request import ChatCompletionRequest
from .chat_completion_request_logit_bias_type_0 import ChatCompletionRequestLogitBiasType0
from .chat_completion_request_messages_item import ChatCompletionRequestMessagesItem
from .chat_completion_request_metadata_type_0 import ChatCompletionRequestMetadataType0
from .chat_completion_request_response_format_type_0 import ChatCompletionRequestResponseFormatType0
from .chat_completion_request_stream_options_type_0 import ChatCompletionRequestStreamOptionsType0
from .chat_completion_request_tool_choice_type_1 import ChatCompletionRequestToolChoiceType1
from .chat_completion_request_tools_type_0_item import ChatCompletionRequestToolsType0Item
from .chunk import Chunk
from .chunk_metadata import ChunkMetadata
from .chunker_config import ChunkerConfig
from .chunker_config_tokenizer_type import ChunkerConfigTokenizerType
from .citation_data import CitationData
from .code_agent_config import CodeAgentConfig
from .code_agent_config_api_type import CodeAgentConfigApiType
from .compaction_tool import CompactionTool
from .compaction_tool_args import CompactionToolArgs
from .compaction_tool_tool_responses import CompactionToolToolResponses
from .config_delete_response import ConfigDeleteResponse
from .config_save_response import ConfigSaveResponse
from .config_update_data import ConfigUpdateData
from .config_version_info import ConfigVersionInfo
from .config_versions_response import ConfigVersionsResponse
from .connection_closed_message import ConnectionClosedMessage
from .consolidated_health_response import ConsolidatedHealthResponse
from .contact_response import ContactResponse
from .contact_response_status import ContactResponseStatus
from .context_usage import ContextUsage
from .conversation_delete_response import ConversationDeleteResponse
from .conversation_response import ConversationResponse
from .conversation_title_update_request import ConversationTitleUpdateRequest
from .conversation_title_update_response import ConversationTitleUpdateResponse
from .copy_document_result import CopyDocumentResult
from .copy_request import CopyRequest
from .copy_response import CopyResponse
from .create_agent_request import CreateAgentRequest
from .create_artifact_detail import CreateArtifactDetail
from .create_doc_tag_request import CreateDocTagRequest
from .create_doc_tag_request_citations_type_0 import CreateDocTagRequestCitationsType0
from .create_plan_detail import CreatePlanDetail
from .create_subscription_request import CreateSubscriptionRequest
from .create_subscription_response import CreateSubscriptionResponse
from .create_tag_request import CreateTagRequest
from .delete_agents_request import DeleteAgentsRequest
from .delete_agents_response_delete_agents import DeleteAgentsResponseDeleteAgents
from .delete_contacts_request import DeleteContactsRequest
from .delete_doc_tag_request import DeleteDocTagRequest
from .delete_documents_request import DeleteDocumentsRequest
from .delete_notifications_request import DeleteNotificationsRequest
from .delete_tag_response import DeleteTagResponse
from .delete_workspaces_request import DeleteWorkspacesRequest
from .doc_metadata import DocMetadata
from .doc_response import DocResponse
from .doc_sim_pair import DocSimPair
from .doc_sim_response import DocSimResponse
from .doc_tag_response import DocTagResponse
from .doc_tag_response_citations_type_0 import DocTagResponseCitationsType0
from .doc_update_request import DocUpdateRequest
from .doctag_llm_config import DoctagLLMConfig
from .doctag_llm_config_api_type import DoctagLLMConfigApiType
from .embedder_config import EmbedderConfig
from .embedder_config_api_type import EmbedderConfigApiType
from .error_message import ErrorMessage
from .evaluation_detail import EvaluationDetail
from .evaluator_llm_config import EvaluatorLLMConfig
from .evaluator_llm_config_api_type import EvaluatorLLMConfigApiType
from .file_delete_response import FileDeleteResponse
from .file_list_response import FileListResponse
from .file_object import FileObject
from .filter_plan_llm_config import FilterPlanLLMConfig
from .filter_plan_llm_config_api_type import FilterPlanLLMConfigApiType
from .generate_annotations_request import GenerateAnnotationsRequest
from .generate_annotations_response import GenerateAnnotationsResponse
from .get_table_of_contents_detail import GetTableOfContentsDetail
from .http_validation_error import HTTPValidationError
from .keyword_embedder_config import KeywordEmbedderConfig
from .login_request import LoginRequest
from .login_response import LoginResponse
from .login_response_workspaces_item import LoginResponseWorkspacesItem
from .logout_response import LogoutResponse
from .mcp_tool_info import McpToolInfo
from .mcp_tools_response import McpToolsResponse
from .memory_llm_config import MemoryLLMConfig
from .memory_llm_config_api_type import MemoryLLMConfigApiType
from .message_delete_response import MessageDeleteResponse
from .message_input import MessageInput
from .message_input_tools import MessageInputTools
from .message_metadata_payload import MessageMetadataPayload
from .message_metadata_payload_role import MessageMetadataPayloadRole
from .message_metadata_payload_tools import MessageMetadataPayloadTools
from .message_queued_event import MessageQueuedEvent
from .message_recipient import MessageRecipient
from .message_response import MessageResponse
from .message_response_role import MessageResponseRole
from .message_response_tools import MessageResponseTools
from .model_citation_config import ModelCitationConfig
from .model_citation_tool import ModelCitationTool
from .model_citation_tool_tool_responses import ModelCitationToolToolResponses
from .model_info import ModelInfo
from .model_status import ModelStatus
from .models_health_status import ModelsHealthStatus
from .models_response import ModelsResponse
from .non_developer_config import NonDeveloperConfig
from .non_developer_config_agents import NonDeveloperConfigAgents
from .notification_response import NotificationResponse
from .notification_type import NotificationType
from .notification_update import NotificationUpdate
from .output_message import OutputMessage
from .output_message_status import OutputMessageStatus
from .output_text_content import OutputTextContent
from .output_tokens_details import OutputTokensDetails
from .parsed_doc import ParsedDoc
from .parsed_doc_footnotes import ParsedDocFootnotes
from .parsed_doc_metadata import ParsedDocMetadata
from .parsed_stage import ParsedStage
from .parser_config import ParserConfig
from .personal_agent_detail import PersonalAgentDetail
from .personal_agent_tool import PersonalAgentTool
from .personal_agent_tool_args import PersonalAgentToolArgs
from .personal_agent_tool_tool_responses import PersonalAgentToolToolResponses
from .planning_llm_config import PlanningLLMConfig
from .planning_llm_config_api_type import PlanningLLMConfigApiType
from .presence_update_message import PresenceUpdateMessage
from .presence_update_message_status import PresenceUpdateMessageStatus
from .product_price import ProductPrice
from .product_response import ProductResponse
from .query_llm_config import QueryLLMConfig
from .query_llm_config_api_type import QueryLLMConfigApiType
from .read_document_detail import ReadDocumentDetail
from .read_url_detail import ReadUrlDetail
from .register_request import RegisterRequest
from .remove_workspace_user_item import RemoveWorkspaceUserItem
from .remove_workspace_users_request import RemoveWorkspaceUsersRequest
from .reranker_config import RerankerConfig
from .reranker_config_api_type import RerankerConfigApiType
from .response_complete_message import ResponseCompleteMessage
from .response_complete_message_status import ResponseCompleteMessageStatus
from .response_completed_event import ResponseCompletedEvent
from .response_completed_payload import ResponseCompletedPayload
from .response_content_part_added_event import ResponseContentPartAddedEvent
from .response_created_event import ResponseCreatedEvent
from .response_created_payload import ResponseCreatedPayload
from .response_error import ResponseError
from .response_failed_event import ResponseFailedEvent
from .response_failed_payload import ResponseFailedPayload
from .response_output_item_added_event import ResponseOutputItemAddedEvent
from .response_output_item_done_event import ResponseOutputItemDoneEvent
from .response_output_text_delta_event import ResponseOutputTextDeltaEvent
from .response_output_text_done_event import ResponseOutputTextDoneEvent
from .response_usage import ResponseUsage
from .responses_api_output_message import ResponsesAPIOutputMessage
from .responses_api_output_message_content_item_type_1 import ResponsesAPIOutputMessageContentItemType1
from .responses_api_output_message_status import ResponsesAPIOutputMessageStatus
from .responses_api_output_text import ResponsesAPIOutputText
from .responses_api_output_text_annotations_item import ResponsesAPIOutputTextAnnotationsItem
from .responses_api_response import ResponsesAPIResponse
from .responses_api_response_error_type_0 import ResponsesAPIResponseErrorType0
from .responses_api_response_metadata_type_0 import ResponsesAPIResponseMetadataType0
from .responses_api_response_status import ResponsesAPIResponseStatus
from .responses_api_usage import ResponsesAPIUsage
from .retrieval_chunk_tool import RetrievalChunkTool
from .retrieval_chunk_tool_args import RetrievalChunkToolArgs
from .retrieval_chunk_tool_args_search_mode_type_0 import RetrievalChunkToolArgsSearchModeType0
from .retrieval_chunk_tool_tool_responses import RetrievalChunkToolToolResponses
from .retrieval_full_context_tool import RetrievalFullContextTool
from .retrieval_full_context_tool_args import RetrievalFullContextToolArgs
from .retrieval_full_context_tool_tool_responses import RetrievalFullContextToolToolResponses
from .retrieval_toc_tool import RetrievalTOCTool
from .retrieval_toc_tool_args import RetrievalTOCToolArgs
from .retrieval_toc_tool_tool_responses import RetrievalTOCToolToolResponses
from .retrieval_toc_tool_tool_responses_additional_property_item import RetrievalTOCToolToolResponsesAdditionalPropertyItem
from .retriever_config import RetrieverConfig
from .review_llm_config import ReviewLLMConfig
from .review_llm_config_api_type import ReviewLLMConfigApiType
from .run_code_config import RunCodeConfig
from .run_code_detail import RunCodeDetail
from .save_skill_detail import SaveSkillDetail
from .search_inside_documents_detail import SearchInsideDocumentsDetail
from .search_mode import SearchMode
from .send_message_request import SendMessageRequest
from .service_status import ServiceStatus
from .service_status_service_info_type_0 import ServiceStatusServiceInfoType0
from .session_info_response import SessionInfoResponse
from .share_conversation_response import ShareConversationResponse
from .sse_schemas import SSESchemas
from .sso_config_response import SSOConfigResponse
from .sso_send_verification_email_request import SSOSendVerificationEmailRequest
from .sso_send_verification_email_response import SSOSendVerificationEmailResponse
from .sso_status_request import SSOStatusRequest
from .sso_status_response import SSOStatusResponse
from .step_usage import StepUsage
from .stream_events_tool import StreamEventsTool
from .stream_events_tool_events_item import StreamEventsToolEventsItem
from .subscription_info import SubscriptionInfo
from .subscription_status_response import SubscriptionStatusResponse
from .summarise_llm_config import SummariseLLMConfig
from .summarise_llm_config_api_type import SummariseLLMConfigApiType
from .table_view import TableView
from .tag_format import TagFormat
from .tag_format_type import TagFormatType
from .tag_response import TagResponse
from .tag_template import TagTemplate
from .task_update_message import TaskUpdateMessage
from .task_update_message_status import TaskUpdateMessageStatus
from .thread import Thread
from .threads_response import ThreadsResponse
from .title_llm_config import TitleLLMConfig
from .title_llm_config_api_type import TitleLLMConfigApiType
from .token_budget_context import TokenBudgetContext
from .token_budget_context_system_detail_type_0 import TokenBudgetContextSystemDetailType0
from .tool_progress_detail import ToolProgressDetail
from .trace_tool import TraceTool
from .trace_tool_steps_item import TraceToolStepsItem
from .trial_update import TrialUpdate
from .update_doc_tag_request import UpdateDocTagRequest
from .update_doc_tag_request_citations_type_0 import UpdateDocTagRequestCitationsType0
from .update_documents_request import UpdateDocumentsRequest
from .update_notifications_request import UpdateNotificationsRequest
from .update_tag_request import UpdateTagRequest
from .update_workspace_user_roles_request import UpdateWorkspaceUserRolesRequest
from .upload_documents_body import UploadDocumentsBody
from .upload_documents_response import UploadDocumentsResponse
from .upload_file_body import UploadFileBody
from .user_input_body import UserInputBody
from .user_input_request_event import UserInputRequestEvent
from .user_message_event import UserMessageEvent
from .user_message_event_role import UserMessageEventRole
from .user_message_event_tools import UserMessageEventTools
from .user_response import UserResponse
from .user_settings_response import UserSettingsResponse
from .user_settings_update import UserSettingsUpdate
from .validation_error import ValidationError
from .validation_error_context import ValidationErrorContext
from .verify_email_request import VerifyEmailRequest
from .verify_email_response import VerifyEmailResponse
from .view_document_pages_detail import ViewDocumentPagesDetail
from .vision_llm_config import VisionLLMConfig
from .vision_llm_config_api_type import VisionLLMConfigApiType
from .web_search_config import WebSearchConfig
from .web_search_detail import WebSearchDetail
from .web_socket_schemas import WebSocketSchemas
from .workspace_create_request import WorkspaceCreateRequest
from .workspace_open_request import WorkspaceOpenRequest
from .workspace_open_response import WorkspaceOpenResponse
from .workspace_response import WorkspaceResponse
from .workspace_role import WorkspaceRole
from .workspace_update_request import WorkspaceUpdateRequest
from .workspace_user_response import WorkspaceUserResponse

__all__ = (
    "AddContactsRequest",
    "AddWorkspaceUsersRequest",
    "AgentsConfig",
    "AgentsConfigAgentApiType",
    "AgentStepEvent",
    "AgentStepEventDetailType0Item",
    "AllConfigs",
    "AllLlmCallsAggregate",
    "ArtifactEvent",
    "ArtifactEventMode",
    "AskUserDetail",
    "AuthMessage",
    "AuthResultMessage",
    "BatchCompleteMessage",
    "BatchCompleteMessageBatchType",
    "ChangePasswordRequest",
    "ChangePasswordRequestRewrappedAgentRecoveryKeysType0",
    "ChangePasswordRequestRewrappedWorkspaceKeys",
    "ChangePasswordResponse",
    "ChatCompletionRequest",
    "ChatCompletionRequestLogitBiasType0",
    "ChatCompletionRequestMessagesItem",
    "ChatCompletionRequestMetadataType0",
    "ChatCompletionRequestResponseFormatType0",
    "ChatCompletionRequestStreamOptionsType0",
    "ChatCompletionRequestToolChoiceType1",
    "ChatCompletionRequestToolsType0Item",
    "Chunk",
    "ChunkerConfig",
    "ChunkerConfigTokenizerType",
    "ChunkMetadata",
    "CitationData",
    "CodeAgentConfig",
    "CodeAgentConfigApiType",
    "CompactionTool",
    "CompactionToolArgs",
    "CompactionToolToolResponses",
    "ConfigDeleteResponse",
    "ConfigSaveResponse",
    "ConfigUpdateData",
    "ConfigVersionInfo",
    "ConfigVersionsResponse",
    "ConnectionClosedMessage",
    "ConsolidatedHealthResponse",
    "ContactResponse",
    "ContactResponseStatus",
    "ContextUsage",
    "ConversationDeleteResponse",
    "ConversationResponse",
    "ConversationTitleUpdateRequest",
    "ConversationTitleUpdateResponse",
    "CopyDocumentResult",
    "CopyRequest",
    "CopyResponse",
    "CreateAgentRequest",
    "CreateArtifactDetail",
    "CreateDocTagRequest",
    "CreateDocTagRequestCitationsType0",
    "CreatePlanDetail",
    "CreateSubscriptionRequest",
    "CreateSubscriptionResponse",
    "CreateTagRequest",
    "DeleteAgentsRequest",
    "DeleteAgentsResponseDeleteAgents",
    "DeleteContactsRequest",
    "DeleteDocTagRequest",
    "DeleteDocumentsRequest",
    "DeleteNotificationsRequest",
    "DeleteTagResponse",
    "DeleteWorkspacesRequest",
    "DocMetadata",
    "DocResponse",
    "DocSimPair",
    "DocSimResponse",
    "DoctagLLMConfig",
    "DoctagLLMConfigApiType",
    "DocTagResponse",
    "DocTagResponseCitationsType0",
    "DocUpdateRequest",
    "EmbedderConfig",
    "EmbedderConfigApiType",
    "ErrorMessage",
    "EvaluationDetail",
    "EvaluatorLLMConfig",
    "EvaluatorLLMConfigApiType",
    "FileDeleteResponse",
    "FileListResponse",
    "FileObject",
    "FilterPlanLLMConfig",
    "FilterPlanLLMConfigApiType",
    "GenerateAnnotationsRequest",
    "GenerateAnnotationsResponse",
    "GetTableOfContentsDetail",
    "HTTPValidationError",
    "KeywordEmbedderConfig",
    "LoginRequest",
    "LoginResponse",
    "LoginResponseWorkspacesItem",
    "LogoutResponse",
    "McpToolInfo",
    "McpToolsResponse",
    "MemoryLLMConfig",
    "MemoryLLMConfigApiType",
    "MessageDeleteResponse",
    "MessageInput",
    "MessageInputTools",
    "MessageMetadataPayload",
    "MessageMetadataPayloadRole",
    "MessageMetadataPayloadTools",
    "MessageQueuedEvent",
    "MessageRecipient",
    "MessageResponse",
    "MessageResponseRole",
    "MessageResponseTools",
    "ModelCitationConfig",
    "ModelCitationTool",
    "ModelCitationToolToolResponses",
    "ModelInfo",
    "ModelsHealthStatus",
    "ModelsResponse",
    "ModelStatus",
    "NonDeveloperConfig",
    "NonDeveloperConfigAgents",
    "NotificationResponse",
    "NotificationType",
    "NotificationUpdate",
    "OutputMessage",
    "OutputMessageStatus",
    "OutputTextContent",
    "OutputTokensDetails",
    "ParsedDoc",
    "ParsedDocFootnotes",
    "ParsedDocMetadata",
    "ParsedStage",
    "ParserConfig",
    "PersonalAgentDetail",
    "PersonalAgentTool",
    "PersonalAgentToolArgs",
    "PersonalAgentToolToolResponses",
    "PlanningLLMConfig",
    "PlanningLLMConfigApiType",
    "PresenceUpdateMessage",
    "PresenceUpdateMessageStatus",
    "ProductPrice",
    "ProductResponse",
    "QueryLLMConfig",
    "QueryLLMConfigApiType",
    "ReadDocumentDetail",
    "ReadUrlDetail",
    "RegisterRequest",
    "RemoveWorkspaceUserItem",
    "RemoveWorkspaceUsersRequest",
    "RerankerConfig",
    "RerankerConfigApiType",
    "ResponseCompletedEvent",
    "ResponseCompletedPayload",
    "ResponseCompleteMessage",
    "ResponseCompleteMessageStatus",
    "ResponseContentPartAddedEvent",
    "ResponseCreatedEvent",
    "ResponseCreatedPayload",
    "ResponseError",
    "ResponseFailedEvent",
    "ResponseFailedPayload",
    "ResponseOutputItemAddedEvent",
    "ResponseOutputItemDoneEvent",
    "ResponseOutputTextDeltaEvent",
    "ResponseOutputTextDoneEvent",
    "ResponsesAPIOutputMessage",
    "ResponsesAPIOutputMessageContentItemType1",
    "ResponsesAPIOutputMessageStatus",
    "ResponsesAPIOutputText",
    "ResponsesAPIOutputTextAnnotationsItem",
    "ResponsesAPIResponse",
    "ResponsesAPIResponseErrorType0",
    "ResponsesAPIResponseMetadataType0",
    "ResponsesAPIResponseStatus",
    "ResponsesAPIUsage",
    "ResponseUsage",
    "RetrievalChunkTool",
    "RetrievalChunkToolArgs",
    "RetrievalChunkToolArgsSearchModeType0",
    "RetrievalChunkToolToolResponses",
    "RetrievalFullContextTool",
    "RetrievalFullContextToolArgs",
    "RetrievalFullContextToolToolResponses",
    "RetrievalTOCTool",
    "RetrievalTOCToolArgs",
    "RetrievalTOCToolToolResponses",
    "RetrievalTOCToolToolResponsesAdditionalPropertyItem",
    "RetrieverConfig",
    "ReviewLLMConfig",
    "ReviewLLMConfigApiType",
    "RunCodeConfig",
    "RunCodeDetail",
    "SaveSkillDetail",
    "SearchInsideDocumentsDetail",
    "SearchMode",
    "SendMessageRequest",
    "ServiceStatus",
    "ServiceStatusServiceInfoType0",
    "SessionInfoResponse",
    "ShareConversationResponse",
    "SSESchemas",
    "SSOConfigResponse",
    "SSOSendVerificationEmailRequest",
    "SSOSendVerificationEmailResponse",
    "SSOStatusRequest",
    "SSOStatusResponse",
    "StepUsage",
    "StreamEventsTool",
    "StreamEventsToolEventsItem",
    "SubscriptionInfo",
    "SubscriptionStatusResponse",
    "SummariseLLMConfig",
    "SummariseLLMConfigApiType",
    "TableView",
    "TagFormat",
    "TagFormatType",
    "TagResponse",
    "TagTemplate",
    "TaskUpdateMessage",
    "TaskUpdateMessageStatus",
    "Thread",
    "ThreadsResponse",
    "TitleLLMConfig",
    "TitleLLMConfigApiType",
    "TokenBudgetContext",
    "TokenBudgetContextSystemDetailType0",
    "ToolProgressDetail",
    "TraceTool",
    "TraceToolStepsItem",
    "TrialUpdate",
    "UpdateDocTagRequest",
    "UpdateDocTagRequestCitationsType0",
    "UpdateDocumentsRequest",
    "UpdateNotificationsRequest",
    "UpdateTagRequest",
    "UpdateWorkspaceUserRolesRequest",
    "UploadDocumentsBody",
    "UploadDocumentsResponse",
    "UploadFileBody",
    "UserInputBody",
    "UserInputRequestEvent",
    "UserMessageEvent",
    "UserMessageEventRole",
    "UserMessageEventTools",
    "UserResponse",
    "UserSettingsResponse",
    "UserSettingsUpdate",
    "ValidationError",
    "ValidationErrorContext",
    "VerifyEmailRequest",
    "VerifyEmailResponse",
    "ViewDocumentPagesDetail",
    "VisionLLMConfig",
    "VisionLLMConfigApiType",
    "WebSearchConfig",
    "WebSearchDetail",
    "WebSocketSchemas",
    "WorkspaceCreateRequest",
    "WorkspaceOpenRequest",
    "WorkspaceOpenResponse",
    "WorkspaceResponse",
    "WorkspaceRole",
    "WorkspaceUpdateRequest",
    "WorkspaceUserResponse",
)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.memory_llm_config_api_type import MemoryLLMConfigApiType
from ..types import UNSET, Unset






T = TypeVar("T", bound="MemoryLLMConfig")



@_attrs_define
class MemoryLLMConfig:
    r""" Configuration for MemoryLLM - synthesizes memory documents (skills, memories, reports).

    Produces markdown documents from gathered context (learnings, retrieval results,
    conversation history). Output is saved as a document through the standard
    upload pipeline with wp_type indicating the document category.

        Attributes:
            api_type (MemoryLLMConfigApiType | Unset): The inference type (local or remote). Default:
                MemoryLLMConfigApiType.REMOTE.
            enable_thinking (bool | Unset): Enable reasoning/thinking mode. Default: False.
            model_name (str | Unset): The model for memory synthesis. Defaults to reasoning model. Default: 'dummy'.
            system_instruction (str | Unset): System instruction for memory classification and synthesis. Default: 'You are
                a knowledge synthesizer. You receive the full agent scratchpad from a completed conversation and decide what, if
                anything, is worth saving.\n\n## Work Product Types\n- **\\"memory\\"**: Facts, findings, reference data — for
                *looking up* information.\n- **\\"skill\\"**: Procedures, workflows, step-by-step instructions — for *doing*
                something.\n\nWhen ambiguous, default to memory.\n\n## Rules\n- Return an empty work_products array if the
                conversation is trivial or produced nothing substantive.\n- Each work product must cover ONE coherent topic.
                Never combine disparate subjects into a single document — create separate documents for each distinct topic.\n-
                Ground everything in the provided context. Do not invent information.\n- Be concise. These are reference
                documents, not essays.\n\n## Output Format\nJSON with a `work_products` array. Each item has `wp_type`, `title`,
                and `content` (markdown).\n\nMemory content: `# Title`, date, source, key findings as bullets, source
                documents.\nSkill content: `# Title`, prerequisites, numbered steps, when to apply.'.
            temperature (float | Unset): Temperature for synthesis. Default: 0.3.
            max_tokens (int | Unset): Maximum tokens for memory content. Default: 8000.
            max_char_context (int | Unset): Maximum characters of input context. Default: 100000.
            max_concurrent (int | Unset): Maximum concurrent synthesis operations. Default: 5.
     """

    api_type: MemoryLLMConfigApiType | Unset = MemoryLLMConfigApiType.REMOTE
    enable_thinking: bool | Unset = False
    model_name: str | Unset = 'dummy'
    system_instruction: str | Unset = 'You are a knowledge synthesizer. You receive the full agent scratchpad from a completed conversation and decide what, if anything, is worth saving.\n\n## Work Product Types\n- **\\"memory\\"**: Facts, findings, reference data — for *looking up* information.\n- **\\"skill\\"**: Procedures, workflows, step-by-step instructions — for *doing* something.\n\nWhen ambiguous, default to memory.\n\n## Rules\n- Return an empty work_products array if the conversation is trivial or produced nothing substantive.\n- Each work product must cover ONE coherent topic. Never combine disparate subjects into a single document — create separate documents for each distinct topic.\n- Ground everything in the provided context. Do not invent information.\n- Be concise. These are reference documents, not essays.\n\n## Output Format\nJSON with a `work_products` array. Each item has `wp_type`, `title`, and `content` (markdown).\n\nMemory content: `# Title`, date, source, key findings as bullets, source documents.\nSkill content: `# Title`, prerequisites, numbered steps, when to apply.'
    temperature: float | Unset = 0.3
    max_tokens: int | Unset = 8000
    max_char_context: int | Unset = 100000
    max_concurrent: int | Unset = 5
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        api_type: str | Unset = UNSET
        if not isinstance(self.api_type, Unset):
            api_type = self.api_type.value


        enable_thinking = self.enable_thinking

        model_name = self.model_name

        system_instruction = self.system_instruction

        temperature = self.temperature

        max_tokens = self.max_tokens

        max_char_context = self.max_char_context

        max_concurrent = self.max_concurrent


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if api_type is not UNSET:
            field_dict["API_TYPE"] = api_type
        if enable_thinking is not UNSET:
            field_dict["ENABLE_THINKING"] = enable_thinking
        if model_name is not UNSET:
            field_dict["MODEL_NAME"] = model_name
        if system_instruction is not UNSET:
            field_dict["SYSTEM_INSTRUCTION"] = system_instruction
        if temperature is not UNSET:
            field_dict["TEMPERATURE"] = temperature
        if max_tokens is not UNSET:
            field_dict["MAX_TOKENS"] = max_tokens
        if max_char_context is not UNSET:
            field_dict["MAX_CHAR_CONTEXT"] = max_char_context
        if max_concurrent is not UNSET:
            field_dict["MAX_CONCURRENT"] = max_concurrent

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _api_type = d.pop("API_TYPE", UNSET)
        api_type: MemoryLLMConfigApiType | Unset
        if isinstance(_api_type,  Unset):
            api_type = UNSET
        else:
            api_type = MemoryLLMConfigApiType(_api_type)




        enable_thinking = d.pop("ENABLE_THINKING", UNSET)

        model_name = d.pop("MODEL_NAME", UNSET)

        system_instruction = d.pop("SYSTEM_INSTRUCTION", UNSET)

        temperature = d.pop("TEMPERATURE", UNSET)

        max_tokens = d.pop("MAX_TOKENS", UNSET)

        max_char_context = d.pop("MAX_CHAR_CONTEXT", UNSET)

        max_concurrent = d.pop("MAX_CONCURRENT", UNSET)

        memory_llm_config = cls(
            api_type=api_type,
            enable_thinking=enable_thinking,
            model_name=model_name,
            system_instruction=system_instruction,
            temperature=temperature,
            max_tokens=max_tokens,
            max_char_context=max_char_context,
            max_concurrent=max_concurrent,
        )


        memory_llm_config.additional_properties = d
        return memory_llm_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset







T = TypeVar("T", bound="ProductPrice")



@_attrs_define
class ProductPrice:
    """ Stripe product price information.

        Attributes:
            price_id (str):
            amount (int):
            currency (str):
            interval (str):
            interval_count (int):
            default_price (bool):
     """

    price_id: str
    amount: int
    currency: str
    interval: str
    interval_count: int
    default_price: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        price_id = self.price_id

        amount = self.amount

        currency = self.currency

        interval = self.interval

        interval_count = self.interval_count

        default_price = self.default_price


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "price_id": price_id,
            "amount": amount,
            "currency": currency,
            "interval": interval,
            "interval_count": interval_count,
            "default_price": default_price,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        price_id = d.pop("price_id")

        amount = d.pop("amount")

        currency = d.pop("currency")

        interval = d.pop("interval")

        interval_count = d.pop("interval_count")

        default_price = d.pop("default_price")

        product_price = cls(
            price_id=price_id,
            amount=amount,
            currency=currency,
            interval=interval,
            interval_count=interval_count,
            default_price=default_price,
        )


        product_price.additional_properties = d
        return product_price

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.doc_tag_response import DocTagResponse
from ...models.http_validation_error import HTTPValidationError
from ...models.update_doc_tag_request import UpdateDocTagRequest
from typing import cast



def _get_kwargs(
    *,
    body: UpdateDocTagRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "patch",
        "url": "/v1/document/doctag",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> DocTagResponse | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = DocTagResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[DocTagResponse | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: UpdateDocTagRequest,

) -> Response[DocTagResponse | HTTPValidationError]:
    """ Update Doctag

     Update a doctag's note or citations.

    Args:
        body (UpdateDocTagRequest): Update a doctag's note or citations. Identified by
            (tag_ext_id, doc_ext_id) pair.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DocTagResponse | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: UpdateDocTagRequest,

) -> DocTagResponse | HTTPValidationError | None:
    """ Update Doctag

     Update a doctag's note or citations.

    Args:
        body (UpdateDocTagRequest): Update a doctag's note or citations. Identified by
            (tag_ext_id, doc_ext_id) pair.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DocTagResponse | HTTPValidationError
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: UpdateDocTagRequest,

) -> Response[DocTagResponse | HTTPValidationError]:
    """ Update Doctag

     Update a doctag's note or citations.

    Args:
        body (UpdateDocTagRequest): Update a doctag's note or citations. Identified by
            (tag_ext_id, doc_ext_id) pair.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DocTagResponse | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: UpdateDocTagRequest,

) -> DocTagResponse | HTTPValidationError | None:
    """ Update Doctag

     Update a doctag's note or citations.

    Args:
        body (UpdateDocTagRequest): Update a doctag's note or citations. Identified by
            (tag_ext_id, doc_ext_id) pair.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DocTagResponse | HTTPValidationError
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.login_request import LoginRequest
from ...models.login_response import LoginResponse
from typing import cast



def _get_kwargs(
    *,
    body: LoginRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/user/login",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | LoginResponse | None:
    if response.status_code == 200:
        response_200 = LoginResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | LoginResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: LoginRequest,

) -> Response[HTTPValidationError | LoginResponse]:
    r""" Login User

     Login with Ed25519 signature verification (local or SSO).

    Authentication flow:
    1. Client derives Ed25519 keypair from password
    2. Client signs \"email|timestamp\" with Ed25519 private key
    3. Server verifies signature using stored Ed25519 public key
    4. Server encrypts response with stored X25519 public key

    For SSO users: Also validates SSO token before proceeding.

    Returns encrypted login response that only the correct password can decrypt.

    Args:
        body (LoginRequest): Unified login request for users and agents.

            Both human users and persistent agents authenticate via email + Ed25519 signature.
            Agents have a synthetic email (``parent+agentname@domain``).

            For local users: email + signature + timestamp
            For SSO users: email + signature + timestamp + sso_token
            For agents: email (agent's synthetic address) + signature + timestamp

            Authentication flow:
            1. Client derives Ed25519 keypair from password (users) or has stored key (agents)
            2. Client signs "email|timestamp" with Ed25519 private key
            3. Server verifies signature using stored Ed25519 public key

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | LoginResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: LoginRequest,

) -> HTTPValidationError | LoginResponse | None:
    r""" Login User

     Login with Ed25519 signature verification (local or SSO).

    Authentication flow:
    1. Client derives Ed25519 keypair from password
    2. Client signs \"email|timestamp\" with Ed25519 private key
    3. Server verifies signature using stored Ed25519 public key
    4. Server encrypts response with stored X25519 public key

    For SSO users: Also validates SSO token before proceeding.

    Returns encrypted login response that only the correct password can decrypt.

    Args:
        body (LoginRequest): Unified login request for users and agents.

            Both human users and persistent agents authenticate via email + Ed25519 signature.
            Agents have a synthetic email (``parent+agentname@domain``).

            For local users: email + signature + timestamp
            For SSO users: email + signature + timestamp + sso_token
            For agents: email (agent's synthetic address) + signature + timestamp

            Authentication flow:
            1. Client derives Ed25519 keypair from password (users) or has stored key (agents)
            2. Client signs "email|timestamp" with Ed25519 private key
            3. Server verifies signature using stored Ed25519 public key

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | LoginResponse
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: LoginRequest,

) -> Response[HTTPValidationError | LoginResponse]:
    r""" Login User

     Login with Ed25519 signature verification (local or SSO).

    Authentication flow:
    1. Client derives Ed25519 keypair from password
    2. Client signs \"email|timestamp\" with Ed25519 private key
    3. Server verifies signature using stored Ed25519 public key
    4. Server encrypts response with stored X25519 public key

    For SSO users: Also validates SSO token before proceeding.

    Returns encrypted login response that only the correct password can decrypt.

    Args:
        body (LoginRequest): Unified login request for users and agents.

            Both human users and persistent agents authenticate via email + Ed25519 signature.
            Agents have a synthetic email (``parent+agentname@domain``).

            For local users: email + signature + timestamp
            For SSO users: email + signature + timestamp + sso_token
            For agents: email (agent's synthetic address) + signature + timestamp

            Authentication flow:
            1. Client derives Ed25519 keypair from password (users) or has stored key (agents)
            2. Client signs "email|timestamp" with Ed25519 private key
            3. Server verifies signature using stored Ed25519 public key

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | LoginResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: LoginRequest,

) -> HTTPValidationError | LoginResponse | None:
    r""" Login User

     Login with Ed25519 signature verification (local or SSO).

    Authentication flow:
    1. Client derives Ed25519 keypair from password
    2. Client signs \"email|timestamp\" with Ed25519 private key
    3. Server verifies signature using stored Ed25519 public key
    4. Server encrypts response with stored X25519 public key

    For SSO users: Also validates SSO token before proceeding.

    Returns encrypted login response that only the correct password can decrypt.

    Args:
        body (LoginRequest): Unified login request for users and agents.

            Both human users and persistent agents authenticate via email + Ed25519 signature.
            Agents have a synthetic email (``parent+agentname@domain``).

            For local users: email + signature + timestamp
            For SSO users: email + signature + timestamp + sso_token
            For agents: email (agent's synthetic address) + signature + timestamp

            Authentication flow:
            1. Client derives Ed25519 keypair from password (users) or has stored key (agents)
            2. Client signs "email|timestamp" with Ed25519 private key
            3. Server verifies signature using stored Ed25519 public key

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | LoginResponse
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime

if TYPE_CHECKING:
  from ..models.tag_format import TagFormat





T = TypeVar("T", bound="TagResponse")



@_attrs_define
class TagResponse:
    """ 
        Attributes:
            external_id (str):
            workspace_ext_id (str):
            name (str):
            tag_type (TagFormat): Tag format configuration stored as JSONB.

                Type-specific fields:
                - select: options (list of choices, can be single or multi-select)
                - search: tag name is the query, chunks include relevance scores
                - checkbox, text, number: type only
            shared (bool):
            doctag_count (int):
            created_by_ext_id (str):
            created_at (datetime.datetime):
            updated_at (datetime.datetime):
            instruction (None | str | Unset):
            parent_ext_id (None | str | Unset):
            updated_by_ext_id (None | str | Unset):
     """

    external_id: str
    workspace_ext_id: str
    name: str
    tag_type: TagFormat
    shared: bool
    doctag_count: int
    created_by_ext_id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    instruction: None | str | Unset = UNSET
    parent_ext_id: None | str | Unset = UNSET
    updated_by_ext_id: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.tag_format import TagFormat
        external_id = self.external_id

        workspace_ext_id = self.workspace_ext_id

        name = self.name

        tag_type = self.tag_type.to_dict()

        shared = self.shared

        doctag_count = self.doctag_count

        created_by_ext_id = self.created_by_ext_id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        instruction: None | str | Unset
        if isinstance(self.instruction, Unset):
            instruction = UNSET
        else:
            instruction = self.instruction

        parent_ext_id: None | str | Unset
        if isinstance(self.parent_ext_id, Unset):
            parent_ext_id = UNSET
        else:
            parent_ext_id = self.parent_ext_id

        updated_by_ext_id: None | str | Unset
        if isinstance(self.updated_by_ext_id, Unset):
            updated_by_ext_id = UNSET
        else:
            updated_by_ext_id = self.updated_by_ext_id


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "external_id": external_id,
            "workspace_ext_id": workspace_ext_id,
            "name": name,
            "tag_type": tag_type,
            "shared": shared,
            "doctag_count": doctag_count,
            "created_by_ext_id": created_by_ext_id,
            "created_at": created_at,
            "updated_at": updated_at,
        })
        if instruction is not UNSET:
            field_dict["instruction"] = instruction
        if parent_ext_id is not UNSET:
            field_dict["parent_ext_id"] = parent_ext_id
        if updated_by_ext_id is not UNSET:
            field_dict["updated_by_ext_id"] = updated_by_ext_id

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tag_format import TagFormat
        d = dict(src_dict)
        external_id = d.pop("external_id")

        workspace_ext_id = d.pop("workspace_ext_id")

        name = d.pop("name")

        tag_type = TagFormat.from_dict(d.pop("tag_type"))




        shared = d.pop("shared")

        doctag_count = d.pop("doctag_count")

        created_by_ext_id = d.pop("created_by_ext_id")

        created_at = isoparse(d.pop("created_at"))




        updated_at = isoparse(d.pop("updated_at"))




        def _parse_instruction(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        instruction = _parse_instruction(d.pop("instruction", UNSET))


        def _parse_parent_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        parent_ext_id = _parse_parent_ext_id(d.pop("parent_ext_id", UNSET))


        def _parse_updated_by_ext_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        updated_by_ext_id = _parse_updated_by_ext_id(d.pop("updated_by_ext_id", UNSET))


        tag_response = cls(
            external_id=external_id,
            workspace_ext_id=workspace_ext_id,
            name=name,
            tag_type=tag_type,
            shared=shared,
            doctag_count=doctag_count,
            created_by_ext_id=created_by_ext_id,
            created_at=created_at,
            updated_at=updated_at,
            instruction=instruction,
            parent_ext_id=parent_ext_id,
            updated_by_ext_id=updated_by_ext_id,
        )


        tag_response.additional_properties = d
        return tag_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.conversation_response import ConversationResponse
  from ..models.workspace_response import WorkspaceResponse





T = TypeVar("T", bound="WorkspaceOpenResponse")



@_attrs_define
class WorkspaceOpenResponse:
    """ Response from opening a workspace — workspace data (no new JWT).

        Attributes:
            workspace (WorkspaceResponse):
            conversations (list[ConversationResponse]):
            documents (list[Any]):
            tags (list[Any]):
     """

    workspace: WorkspaceResponse
    conversations: list[ConversationResponse]
    documents: list[Any]
    tags: list[Any]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.workspace_response import WorkspaceResponse
        from ..models.conversation_response import ConversationResponse
        workspace = self.workspace.to_dict()

        conversations = []
        for conversations_item_data in self.conversations:
            conversations_item = conversations_item_data.to_dict()
            conversations.append(conversations_item)



        documents = self.documents



        tags = self.tags




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "workspace": workspace,
            "conversations": conversations,
            "documents": documents,
            "tags": tags,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.conversation_response import ConversationResponse
        from ..models.workspace_response import WorkspaceResponse
        d = dict(src_dict)
        workspace = WorkspaceResponse.from_dict(d.pop("workspace"))




        conversations = []
        _conversations = d.pop("conversations")
        for conversations_item_data in (_conversations):
            conversations_item = ConversationResponse.from_dict(conversations_item_data)



            conversations.append(conversations_item)


        documents = cast(list[Any], d.pop("documents"))


        tags = cast(list[Any], d.pop("tags"))


        workspace_open_response = cls(
            workspace=workspace,
            conversations=conversations,
            documents=documents,
            tags=tags,
        )


        workspace_open_response.additional_properties = d
        return workspace_open_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.file_object import FileObject





T = TypeVar("T", bound="FileListResponse")



@_attrs_define
class FileListResponse:
    """ OpenAI-compatible file list response with cursor pagination.

        Attributes:
            data (list[FileObject]):
            object_ (str | Unset):  Default: 'list'.
            has_more (bool | Unset):  Default: False.
            first_id (None | str | Unset):
            last_id (None | str | Unset):
     """

    data: list[FileObject]
    object_: str | Unset = 'list'
    has_more: bool | Unset = False
    first_id: None | str | Unset = UNSET
    last_id: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.file_object import FileObject
        data = []
        for data_item_data in self.data:
            data_item = data_item_data.to_dict()
            data.append(data_item)



        object_ = self.object_

        has_more = self.has_more

        first_id: None | str | Unset
        if isinstance(self.first_id, Unset):
            first_id = UNSET
        else:
            first_id = self.first_id

        last_id: None | str | Unset
        if isinstance(self.last_id, Unset):
            last_id = UNSET
        else:
            last_id = self.last_id


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "data": data,
        })
        if object_ is not UNSET:
            field_dict["object"] = object_
        if has_more is not UNSET:
            field_dict["has_more"] = has_more
        if first_id is not UNSET:
            field_dict["first_id"] = first_id
        if last_id is not UNSET:
            field_dict["last_id"] = last_id

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.file_object import FileObject
        d = dict(src_dict)
        data = []
        _data = d.pop("data")
        for data_item_data in (_data):
            data_item = FileObject.from_dict(data_item_data)



            data.append(data_item)


        object_ = d.pop("object", UNSET)

        has_more = d.pop("has_more", UNSET)

        def _parse_first_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        first_id = _parse_first_id(d.pop("first_id", UNSET))


        def _parse_last_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        last_id = _parse_last_id(d.pop("last_id", UNSET))


        file_list_response = cls(
            data=data,
            object_=object_,
            has_more=has_more,
            first_id=first_id,
            last_id=last_id,
        )


        file_list_response.additional_properties = d
        return file_list_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.generate_annotations_request import GenerateAnnotationsRequest
from ...models.generate_annotations_response import GenerateAnnotationsResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    *,
    body: GenerateAnnotationsRequest,
    config_ext_id: None | str | Unset = UNSET,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    params: dict[str, Any] = {}

    json_config_ext_id: None | str | Unset
    if isinstance(config_ext_id, Unset):
        json_config_ext_id = UNSET
    else:
        json_config_ext_id = config_ext_id
    params["config_ext_id"] = json_config_ext_id


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/document/doctag/generate",
        "params": params,
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> GenerateAnnotationsResponse | HTTPValidationError | None:
    if response.status_code == 202:
        response_202 = GenerateAnnotationsResponse.from_dict(response.json())



        return response_202

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[GenerateAnnotationsResponse | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: GenerateAnnotationsRequest,
    config_ext_id: None | str | Unset = UNSET,

) -> Response[GenerateAnnotationsResponse | HTTPValidationError]:
    """ Generate Doctags

     Generate AI annotations for documents using tag instructions.

    Creates doctags with AI-generated notes and citations for each (doc, tag) pair.
    Uses tag name + optional instruction as the question to answer about each document.

    Returns 202 Accepted immediately - processing happens in background.
    WebSocket notification sent when complete.

    Args:
        config_ext_id (None | str | Unset): Configuration to use for LLM
        body (GenerateAnnotationsRequest): Request to generate AI annotations for documents using
            tag instructions.

            Creates a matrix of (docs × tags) annotations. Each tag's name is used as
            the instruction/question to answer about each document.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GenerateAnnotationsResponse | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,
config_ext_id=config_ext_id,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: GenerateAnnotationsRequest,
    config_ext_id: None | str | Unset = UNSET,

) -> GenerateAnnotationsResponse | HTTPValidationError | None:
    """ Generate Doctags

     Generate AI annotations for documents using tag instructions.

    Creates doctags with AI-generated notes and citations for each (doc, tag) pair.
    Uses tag name + optional instruction as the question to answer about each document.

    Returns 202 Accepted immediately - processing happens in background.
    WebSocket notification sent when complete.

    Args:
        config_ext_id (None | str | Unset): Configuration to use for LLM
        body (GenerateAnnotationsRequest): Request to generate AI annotations for documents using
            tag instructions.

            Creates a matrix of (docs × tags) annotations. Each tag's name is used as
            the instruction/question to answer about each document.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GenerateAnnotationsResponse | HTTPValidationError
     """


    return sync_detailed(
        client=client,
body=body,
config_ext_id=config_ext_id,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: GenerateAnnotationsRequest,
    config_ext_id: None | str | Unset = UNSET,

) -> Response[GenerateAnnotationsResponse | HTTPValidationError]:
    """ Generate Doctags

     Generate AI annotations for documents using tag instructions.

    Creates doctags with AI-generated notes and citations for each (doc, tag) pair.
    Uses tag name + optional instruction as the question to answer about each document.

    Returns 202 Accepted immediately - processing happens in background.
    WebSocket notification sent when complete.

    Args:
        config_ext_id (None | str | Unset): Configuration to use for LLM
        body (GenerateAnnotationsRequest): Request to generate AI annotations for documents using
            tag instructions.

            Creates a matrix of (docs × tags) annotations. Each tag's name is used as
            the instruction/question to answer about each document.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[GenerateAnnotationsResponse | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        body=body,
config_ext_id=config_ext_id,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: GenerateAnnotationsRequest,
    config_ext_id: None | str | Unset = UNSET,

) -> GenerateAnnotationsResponse | HTTPValidationError | None:
    """ Generate Doctags

     Generate AI annotations for documents using tag instructions.

    Creates doctags with AI-generated notes and citations for each (doc, tag) pair.
    Uses tag name + optional instruction as the question to answer about each document.

    Returns 202 Accepted immediately - processing happens in background.
    WebSocket notification sent when complete.

    Args:
        config_ext_id (None | str | Unset): Configuration to use for LLM
        body (GenerateAnnotationsRequest): Request to generate AI annotations for documents using
            tag instructions.

            Creates a matrix of (docs × tags) annotations. Each tag's name is used as
            the instruction/question to answer about each document.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        GenerateAnnotationsResponse | HTTPValidationError
     """


    return (await asyncio_detailed(
        client=client,
body=body,
config_ext_id=config_ext_id,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast

if TYPE_CHECKING:
  from ..models.message_metadata_payload import MessageMetadataPayload
  from ..models.output_message import OutputMessage
  from ..models.response_usage import ResponseUsage





T = TypeVar("T", bound="ResponseCompletedPayload")



@_attrs_define
class ResponseCompletedPayload:
    """ The ``response`` dict in a ``response.completed`` SSE event.

        Attributes:
            id (str): Response ID (assistant_message_ext_id)
            created_at (int): Unix epoch timestamp when the response was created
            model (str): Model name used for the response
            output (list[OutputMessage]): Final output messages with content
            usage (ResponseUsage): Token usage summary in a completed response (OpenAI format).
            metadata (MessageMetadataPayload): ``message_metadata`` payload within ``response.completed.response.metadata``.

                Contains the full assistant :class:`MessageResponse` plus context_usage and
                optional suggested_query.
            object_ (Literal['response'] | Unset):  Default: 'response'.
            status (Literal['completed'] | Unset):  Default: 'completed'.
     """

    id: str
    created_at: int
    model: str
    output: list[OutputMessage]
    usage: ResponseUsage
    metadata: MessageMetadataPayload
    object_: Literal['response'] | Unset = 'response'
    status: Literal['completed'] | Unset = 'completed'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.output_message import OutputMessage
        from ..models.message_metadata_payload import MessageMetadataPayload
        from ..models.response_usage import ResponseUsage
        id = self.id

        created_at = self.created_at

        model = self.model

        output = []
        for output_item_data in self.output:
            output_item = output_item_data.to_dict()
            output.append(output_item)



        usage = self.usage.to_dict()

        metadata = self.metadata.to_dict()

        object_ = self.object_

        status = self.status


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "id": id,
            "created_at": created_at,
            "model": model,
            "output": output,
            "usage": usage,
            "metadata": metadata,
        })
        if object_ is not UNSET:
            field_dict["object"] = object_
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.message_metadata_payload import MessageMetadataPayload
        from ..models.output_message import OutputMessage
        from ..models.response_usage import ResponseUsage
        d = dict(src_dict)
        id = d.pop("id")

        created_at = d.pop("created_at")

        model = d.pop("model")

        output = []
        _output = d.pop("output")
        for output_item_data in (_output):
            output_item = OutputMessage.from_dict(output_item_data)



            output.append(output_item)


        usage = ResponseUsage.from_dict(d.pop("usage"))




        metadata = MessageMetadataPayload.from_dict(d.pop("metadata"))




        object_ = cast(Literal['response'] | Unset , d.pop("object", UNSET))
        if object_ != 'response'and not isinstance(object_, Unset):
            raise ValueError(f"object must match const 'response', got '{object_}'")

        status = cast(Literal['completed'] | Unset , d.pop("status", UNSET))
        if status != 'completed'and not isinstance(status, Unset):
            raise ValueError(f"status must match const 'completed', got '{status}'")

        response_completed_payload = cls(
            id=id,
            created_at=created_at,
            model=model,
            output=output,
            usage=usage,
            metadata=metadata,
            object_=object_,
            status=status,
        )


        response_completed_payload.additional_properties = d
        return response_completed_payload

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.batch_complete_message_batch_type import BatchCompleteMessageBatchType
from ..types import UNSET, Unset
from typing import cast
from typing import Literal, cast






T = TypeVar("T", bound="BatchCompleteMessage")



@_attrs_define
class BatchCompleteMessage:
    """ Notification that a batch operation (upload or doctag generation) completed.

        Attributes:
            batch_type (BatchCompleteMessageBatchType):
            workspace_ext_id (str):
            doc_ext_ids (list[str]):
            type_ (Literal['batch_complete'] | Unset):  Default: 'batch_complete'.
     """

    batch_type: BatchCompleteMessageBatchType
    workspace_ext_id: str
    doc_ext_ids: list[str]
    type_: Literal['batch_complete'] | Unset = 'batch_complete'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        batch_type = self.batch_type.value

        workspace_ext_id = self.workspace_ext_id

        doc_ext_ids = self.doc_ext_ids



        type_ = self.type_


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "batch_type": batch_type,
            "workspace_ext_id": workspace_ext_id,
            "doc_ext_ids": doc_ext_ids,
        })
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        batch_type = BatchCompleteMessageBatchType(d.pop("batch_type"))




        workspace_ext_id = d.pop("workspace_ext_id")

        doc_ext_ids = cast(list[str], d.pop("doc_ext_ids"))


        type_ = cast(Literal['batch_complete'] | Unset , d.pop("type", UNSET))
        if type_ != 'batch_complete'and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'batch_complete', got '{type_}'")

        batch_complete_message = cls(
            batch_type=batch_type,
            workspace_ext_id=workspace_ext_id,
            doc_ext_ids=doc_ext_ids,
            type_=type_,
        )


        batch_complete_message.additional_properties = d
        return batch_complete_message

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="RetrievalFullContextToolArgs")



@_attrs_define
class RetrievalFullContextToolArgs:
    """ Typed arguments for retrieval full context tool.

        Attributes:
            doc_ext_ids (list[str] | Unset):
            from_ref (None | str | Unset):
            to_ref (None | str | Unset):
     """

    doc_ext_ids: list[str] | Unset = UNSET
    from_ref: None | str | Unset = UNSET
    to_ref: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        doc_ext_ids: list[str] | Unset = UNSET
        if not isinstance(self.doc_ext_ids, Unset):
            doc_ext_ids = self.doc_ext_ids



        from_ref: None | str | Unset
        if isinstance(self.from_ref, Unset):
            from_ref = UNSET
        else:
            from_ref = self.from_ref

        to_ref: None | str | Unset
        if isinstance(self.to_ref, Unset):
            to_ref = UNSET
        else:
            to_ref = self.to_ref


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if doc_ext_ids is not UNSET:
            field_dict["doc_ext_ids"] = doc_ext_ids
        if from_ref is not UNSET:
            field_dict["from_ref"] = from_ref
        if to_ref is not UNSET:
            field_dict["to_ref"] = to_ref

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        doc_ext_ids = cast(list[str], d.pop("doc_ext_ids", UNSET))


        def _parse_from_ref(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        from_ref = _parse_from_ref(d.pop("from_ref", UNSET))


        def _parse_to_ref(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        to_ref = _parse_to_ref(d.pop("to_ref", UNSET))


        retrieval_full_context_tool_args = cls(
            doc_ext_ids=doc_ext_ids,
            from_ref=from_ref,
            to_ref=to_ref,
        )


        retrieval_full_context_tool_args.additional_properties = d
        return retrieval_full_context_tool_args

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.agents_config import AgentsConfig
  from ..models.chunker_config import ChunkerConfig
  from ..models.code_agent_config import CodeAgentConfig
  from ..models.doctag_llm_config import DoctagLLMConfig
  from ..models.embedder_config import EmbedderConfig
  from ..models.evaluator_llm_config import EvaluatorLLMConfig
  from ..models.filter_plan_llm_config import FilterPlanLLMConfig
  from ..models.keyword_embedder_config import KeywordEmbedderConfig
  from ..models.memory_llm_config import MemoryLLMConfig
  from ..models.model_citation_config import ModelCitationConfig
  from ..models.parser_config import ParserConfig
  from ..models.planning_llm_config import PlanningLLMConfig
  from ..models.query_llm_config import QueryLLMConfig
  from ..models.reranker_config import RerankerConfig
  from ..models.retriever_config import RetrieverConfig
  from ..models.review_llm_config import ReviewLLMConfig
  from ..models.summarise_llm_config import SummariseLLMConfig
  from ..models.title_llm_config import TitleLLMConfig
  from ..models.vision_llm_config import VisionLLMConfig





T = TypeVar("T", bound="AllConfigs")



@_attrs_define
class AllConfigs:
    """ 
        Attributes:
            agents (AgentsConfig | Unset): Configuration for agent workflow.
            query_llm (QueryLLMConfig | Unset):
            review_llm (ReviewLLMConfig | Unset): Configuration for ReviewLLM - reviews agent draft answers against source
                material.
            evaluator_llm (EvaluatorLLMConfig | Unset):
            title_llm (TitleLLMConfig | Unset):
            summarise_llm (SummariseLLMConfig | Unset): Configuration for conversation summarisation. Used for compaction
                and general summaries.
            doctag_llm (DoctagLLMConfig | Unset): Configuration for DoctagLLM - extracts information from documents based on
                tag instructions.
            memory_llm (MemoryLLMConfig | Unset): Configuration for MemoryLLM - synthesizes memory documents (skills,
                memories, reports).

                Produces markdown documents from gathered context (learnings, retrieval results,
                conversation history). Output is saved as a document through the standard
                upload pipeline with wp_type indicating the document category.
            planning_llm (PlanningLLMConfig | Unset): Configuration for PlanningLLM — generates research plans on demand.

                Backed by a reasoning model. The agent calls the create_plan tool when it
                deems a task complex enough; the plan text is returned as a tool result.
            filter_plan_llm (FilterPlanLLMConfig | Unset): Configuration for FilterPlanLLM — pre-agentic context selector.

                Runs before the agent loop to select relevant history messages and documents,
                plus produce a concise goal statement that primes the agent.
                Always uses the fast model (LLM_MODEL_NAME). In auto mode, also recommends
                a model tier (fast/wise/premium) for the agent loop.
            vision_llm (VisionLLMConfig | Unset): Configuration for VisionLLM — visually inspects document pages.

                Used as a targeted fallback when text extraction (read_document)
                is insufficient, e.g. for figures, charts, tables, scanned pages.
                The prompt is constructed dynamically per call (no system instruction).
            code_agent (CodeAgentConfig | Unset): Configuration for CodeAgent — sub-agent that writes and executes code via
                sandbox.
            model_citation (ModelCitationConfig | Unset):
            retriever (RetrieverConfig | Unset):
            reranker (RerankerConfig | Unset):
            parser (ParserConfig | Unset):
            chunker (ChunkerConfig | Unset):
            embedder (EmbedderConfig | Unset):
            keyword_embedder (KeywordEmbedderConfig | Unset): Configuration for keyword embedder with BM25 scoring.
     """

    agents: AgentsConfig | Unset = UNSET
    query_llm: QueryLLMConfig | Unset = UNSET
    review_llm: ReviewLLMConfig | Unset = UNSET
    evaluator_llm: EvaluatorLLMConfig | Unset = UNSET
    title_llm: TitleLLMConfig | Unset = UNSET
    summarise_llm: SummariseLLMConfig | Unset = UNSET
    doctag_llm: DoctagLLMConfig | Unset = UNSET
    memory_llm: MemoryLLMConfig | Unset = UNSET
    planning_llm: PlanningLLMConfig | Unset = UNSET
    filter_plan_llm: FilterPlanLLMConfig | Unset = UNSET
    vision_llm: VisionLLMConfig | Unset = UNSET
    code_agent: CodeAgentConfig | Unset = UNSET
    model_citation: ModelCitationConfig | Unset = UNSET
    retriever: RetrieverConfig | Unset = UNSET
    reranker: RerankerConfig | Unset = UNSET
    parser: ParserConfig | Unset = UNSET
    chunker: ChunkerConfig | Unset = UNSET
    embedder: EmbedderConfig | Unset = UNSET
    keyword_embedder: KeywordEmbedderConfig | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.title_llm_config import TitleLLMConfig
        from ..models.filter_plan_llm_config import FilterPlanLLMConfig
        from ..models.embedder_config import EmbedderConfig
        from ..models.vision_llm_config import VisionLLMConfig
        from ..models.doctag_llm_config import DoctagLLMConfig
        from ..models.retriever_config import RetrieverConfig
        from ..models.reranker_config import RerankerConfig
        from ..models.model_citation_config import ModelCitationConfig
        from ..models.evaluator_llm_config import EvaluatorLLMConfig
        from ..models.summarise_llm_config import SummariseLLMConfig
        from ..models.chunker_config import ChunkerConfig
        from ..models.keyword_embedder_config import KeywordEmbedderConfig
        from ..models.query_llm_config import QueryLLMConfig
        from ..models.memory_llm_config import MemoryLLMConfig
        from ..models.planning_llm_config import PlanningLLMConfig
        from ..models.code_agent_config import CodeAgentConfig
        from ..models.review_llm_config import ReviewLLMConfig
        from ..models.agents_config import AgentsConfig
        from ..models.parser_config import ParserConfig
        agents: dict[str, Any] | Unset = UNSET
        if not isinstance(self.agents, Unset):
            agents = self.agents.to_dict()

        query_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.query_llm, Unset):
            query_llm = self.query_llm.to_dict()

        review_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.review_llm, Unset):
            review_llm = self.review_llm.to_dict()

        evaluator_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.evaluator_llm, Unset):
            evaluator_llm = self.evaluator_llm.to_dict()

        title_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.title_llm, Unset):
            title_llm = self.title_llm.to_dict()

        summarise_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.summarise_llm, Unset):
            summarise_llm = self.summarise_llm.to_dict()

        doctag_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.doctag_llm, Unset):
            doctag_llm = self.doctag_llm.to_dict()

        memory_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.memory_llm, Unset):
            memory_llm = self.memory_llm.to_dict()

        planning_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.planning_llm, Unset):
            planning_llm = self.planning_llm.to_dict()

        filter_plan_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.filter_plan_llm, Unset):
            filter_plan_llm = self.filter_plan_llm.to_dict()

        vision_llm: dict[str, Any] | Unset = UNSET
        if not isinstance(self.vision_llm, Unset):
            vision_llm = self.vision_llm.to_dict()

        code_agent: dict[str, Any] | Unset = UNSET
        if not isinstance(self.code_agent, Unset):
            code_agent = self.code_agent.to_dict()

        model_citation: dict[str, Any] | Unset = UNSET
        if not isinstance(self.model_citation, Unset):
            model_citation = self.model_citation.to_dict()

        retriever: dict[str, Any] | Unset = UNSET
        if not isinstance(self.retriever, Unset):
            retriever = self.retriever.to_dict()

        reranker: dict[str, Any] | Unset = UNSET
        if not isinstance(self.reranker, Unset):
            reranker = self.reranker.to_dict()

        parser: dict[str, Any] | Unset = UNSET
        if not isinstance(self.parser, Unset):
            parser = self.parser.to_dict()

        chunker: dict[str, Any] | Unset = UNSET
        if not isinstance(self.chunker, Unset):
            chunker = self.chunker.to_dict()

        embedder: dict[str, Any] | Unset = UNSET
        if not isinstance(self.embedder, Unset):
            embedder = self.embedder.to_dict()

        keyword_embedder: dict[str, Any] | Unset = UNSET
        if not isinstance(self.keyword_embedder, Unset):
            keyword_embedder = self.keyword_embedder.to_dict()


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if agents is not UNSET:
            field_dict["Agents"] = agents
        if query_llm is not UNSET:
            field_dict["QueryLLM"] = query_llm
        if review_llm is not UNSET:
            field_dict["ReviewLLM"] = review_llm
        if evaluator_llm is not UNSET:
            field_dict["EvaluatorLLM"] = evaluator_llm
        if title_llm is not UNSET:
            field_dict["TitleLLM"] = title_llm
        if summarise_llm is not UNSET:
            field_dict["SummariseLLM"] = summarise_llm
        if doctag_llm is not UNSET:
            field_dict["DoctagLLM"] = doctag_llm
        if memory_llm is not UNSET:
            field_dict["MemoryLLM"] = memory_llm
        if planning_llm is not UNSET:
            field_dict["PlanningLLM"] = planning_llm
        if filter_plan_llm is not UNSET:
            field_dict["FilterPlanLLM"] = filter_plan_llm
        if vision_llm is not UNSET:
            field_dict["VisionLLM"] = vision_llm
        if code_agent is not UNSET:
            field_dict["CodeAgent"] = code_agent
        if model_citation is not UNSET:
            field_dict["ModelCitation"] = model_citation
        if retriever is not UNSET:
            field_dict["Retriever"] = retriever
        if reranker is not UNSET:
            field_dict["Reranker"] = reranker
        if parser is not UNSET:
            field_dict["Parser"] = parser
        if chunker is not UNSET:
            field_dict["Chunker"] = chunker
        if embedder is not UNSET:
            field_dict["Embedder"] = embedder
        if keyword_embedder is not UNSET:
            field_dict["KeywordEmbedder"] = keyword_embedder

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.agents_config import AgentsConfig
        from ..models.chunker_config import ChunkerConfig
        from ..models.code_agent_config import CodeAgentConfig
        from ..models.doctag_llm_config import DoctagLLMConfig
        from ..models.embedder_config import EmbedderConfig
        from ..models.evaluator_llm_config import EvaluatorLLMConfig
        from ..models.filter_plan_llm_config import FilterPlanLLMConfig
        from ..models.keyword_embedder_config import KeywordEmbedderConfig
        from ..models.memory_llm_config import MemoryLLMConfig
        from ..models.model_citation_config import ModelCitationConfig
        from ..models.parser_config import ParserConfig
        from ..models.planning_llm_config import PlanningLLMConfig
        from ..models.query_llm_config import QueryLLMConfig
        from ..models.reranker_config import RerankerConfig
        from ..models.retriever_config import RetrieverConfig
        from ..models.review_llm_config import ReviewLLMConfig
        from ..models.summarise_llm_config import SummariseLLMConfig
        from ..models.title_llm_config import TitleLLMConfig
        from ..models.vision_llm_config import VisionLLMConfig
        d = dict(src_dict)
        _agents = d.pop("Agents", UNSET)
        agents: AgentsConfig | Unset
        if isinstance(_agents,  Unset):
            agents = UNSET
        else:
            agents = AgentsConfig.from_dict(_agents)




        _query_llm = d.pop("QueryLLM", UNSET)
        query_llm: QueryLLMConfig | Unset
        if isinstance(_query_llm,  Unset):
            query_llm = UNSET
        else:
            query_llm = QueryLLMConfig.from_dict(_query_llm)




        _review_llm = d.pop("ReviewLLM", UNSET)
        review_llm: ReviewLLMConfig | Unset
        if isinstance(_review_llm,  Unset):
            review_llm = UNSET
        else:
            review_llm = ReviewLLMConfig.from_dict(_review_llm)




        _evaluator_llm = d.pop("EvaluatorLLM", UNSET)
        evaluator_llm: EvaluatorLLMConfig | Unset
        if isinstance(_evaluator_llm,  Unset):
            evaluator_llm = UNSET
        else:
            evaluator_llm = EvaluatorLLMConfig.from_dict(_evaluator_llm)




        _title_llm = d.pop("TitleLLM", UNSET)
        title_llm: TitleLLMConfig | Unset
        if isinstance(_title_llm,  Unset):
            title_llm = UNSET
        else:
            title_llm = TitleLLMConfig.from_dict(_title_llm)




        _summarise_llm = d.pop("SummariseLLM", UNSET)
        summarise_llm: SummariseLLMConfig | Unset
        if isinstance(_summarise_llm,  Unset):
            summarise_llm = UNSET
        else:
            summarise_llm = SummariseLLMConfig.from_dict(_summarise_llm)




        _doctag_llm = d.pop("DoctagLLM", UNSET)
        doctag_llm: DoctagLLMConfig | Unset
        if isinstance(_doctag_llm,  Unset):
            doctag_llm = UNSET
        else:
            doctag_llm = DoctagLLMConfig.from_dict(_doctag_llm)




        _memory_llm = d.pop("MemoryLLM", UNSET)
        memory_llm: MemoryLLMConfig | Unset
        if isinstance(_memory_llm,  Unset):
            memory_llm = UNSET
        else:
            memory_llm = MemoryLLMConfig.from_dict(_memory_llm)




        _planning_llm = d.pop("PlanningLLM", UNSET)
        planning_llm: PlanningLLMConfig | Unset
        if isinstance(_planning_llm,  Unset):
            planning_llm = UNSET
        else:
            planning_llm = PlanningLLMConfig.from_dict(_planning_llm)




        _filter_plan_llm = d.pop("FilterPlanLLM", UNSET)
        filter_plan_llm: FilterPlanLLMConfig | Unset
        if isinstance(_filter_plan_llm,  Unset):
            filter_plan_llm = UNSET
        else:
            filter_plan_llm = FilterPlanLLMConfig.from_dict(_filter_plan_llm)




        _vision_llm = d.pop("VisionLLM", UNSET)
        vision_llm: VisionLLMConfig | Unset
        if isinstance(_vision_llm,  Unset):
            vision_llm = UNSET
        else:
            vision_llm = VisionLLMConfig.from_dict(_vision_llm)




        _code_agent = d.pop("CodeAgent", UNSET)
        code_agent: CodeAgentConfig | Unset
        if isinstance(_code_agent,  Unset):
            code_agent = UNSET
        else:
            code_agent = CodeAgentConfig.from_dict(_code_agent)




        _model_citation = d.pop("ModelCitation", UNSET)
        model_citation: ModelCitationConfig | Unset
        if isinstance(_model_citation,  Unset):
            model_citation = UNSET
        else:
            model_citation = ModelCitationConfig.from_dict(_model_citation)




        _retriever = d.pop("Retriever", UNSET)
        retriever: RetrieverConfig | Unset
        if isinstance(_retriever,  Unset):
            retriever = UNSET
        else:
            retriever = RetrieverConfig.from_dict(_retriever)




        _reranker = d.pop("Reranker", UNSET)
        reranker: RerankerConfig | Unset
        if isinstance(_reranker,  Unset):
            reranker = UNSET
        else:
            reranker = RerankerConfig.from_dict(_reranker)




        _parser = d.pop("Parser", UNSET)
        parser: ParserConfig | Unset
        if isinstance(_parser,  Unset):
            parser = UNSET
        else:
            parser = ParserConfig.from_dict(_parser)




        _chunker = d.pop("Chunker", UNSET)
        chunker: ChunkerConfig | Unset
        if isinstance(_chunker,  Unset):
            chunker = UNSET
        else:
            chunker = ChunkerConfig.from_dict(_chunker)




        _embedder = d.pop("Embedder", UNSET)
        embedder: EmbedderConfig | Unset
        if isinstance(_embedder,  Unset):
            embedder = UNSET
        else:
            embedder = EmbedderConfig.from_dict(_embedder)




        _keyword_embedder = d.pop("KeywordEmbedder", UNSET)
        keyword_embedder: KeywordEmbedderConfig | Unset
        if isinstance(_keyword_embedder,  Unset):
            keyword_embedder = UNSET
        else:
            keyword_embedder = KeywordEmbedderConfig.from_dict(_keyword_embedder)




        all_configs = cls(
            agents=agents,
            query_llm=query_llm,
            review_llm=review_llm,
            evaluator_llm=evaluator_llm,
            title_llm=title_llm,
            summarise_llm=summarise_llm,
            doctag_llm=doctag_llm,
            memory_llm=memory_llm,
            planning_llm=planning_llm,
            filter_plan_llm=filter_plan_llm,
            vision_llm=vision_llm,
            code_agent=code_agent,
            model_citation=model_citation,
            retriever=retriever,
            reranker=reranker,
            parser=parser,
            chunker=chunker,
            embedder=embedder,
            keyword_embedder=keyword_embedder,
        )


        all_configs.additional_properties = d
        return all_configs

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.doc_response import DocResponse





T = TypeVar("T", bound="DocSimPair")



@_attrs_define
class DocSimPair:
    """ A pair of documents identified as near-duplicates by centroid similarity.

        Attributes:
            doc_a (DocResponse):
            doc_b (DocResponse):
            similarity (float):
     """

    doc_a: DocResponse
    doc_b: DocResponse
    similarity: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.doc_response import DocResponse
        doc_a = self.doc_a.to_dict()

        doc_b = self.doc_b.to_dict()

        similarity = self.similarity


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "doc_a": doc_a,
            "doc_b": doc_b,
            "similarity": similarity,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.doc_response import DocResponse
        d = dict(src_dict)
        doc_a = DocResponse.from_dict(d.pop("doc_a"))




        doc_b = DocResponse.from_dict(d.pop("doc_b"))




        similarity = d.pop("similarity")

        doc_sim_pair = cls(
            doc_a=doc_a,
            doc_b=doc_b,
            similarity=similarity,
        )


        doc_sim_pair.additional_properties = d
        return doc_sim_pair

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.all_configs import AllConfigs
from ...models.http_validation_error import HTTPValidationError
from ...models.non_developer_config import NonDeveloperConfig
from typing import cast



def _get_kwargs(
    config_ext_id: str,

) -> dict[str, Any]:
    

    

    

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/configs/{config_ext_id}".format(config_ext_id=quote(str(config_ext_id), safe=""),),
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> AllConfigs | NonDeveloperConfig | HTTPValidationError | None:
    if response.status_code == 200:
        def _parse_response_200(data: object) -> AllConfigs | NonDeveloperConfig:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_0 = AllConfigs.from_dict(data)



                return response_200_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_200_type_1 = NonDeveloperConfig.from_dict(data)



            return response_200_type_1

        response_200 = _parse_response_200(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[AllConfigs | NonDeveloperConfig | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    config_ext_id: str,
    *,
    client: AuthenticatedClient | Client,

) -> Response[AllConfigs | NonDeveloperConfig | HTTPValidationError]:
    """ Get Configs

     Read configurations from database to be displayed in the UI

    Args:
        config_ext_id (str): Config name: 'cfg-XXXXXXXX' or 'default' for system default

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AllConfigs | NonDeveloperConfig | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        config_ext_id=config_ext_id,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    config_ext_id: str,
    *,
    client: AuthenticatedClient | Client,

) -> AllConfigs | NonDeveloperConfig | HTTPValidationError | None:
    """ Get Configs

     Read configurations from database to be displayed in the UI

    Args:
        config_ext_id (str): Config name: 'cfg-XXXXXXXX' or 'default' for system default

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AllConfigs | NonDeveloperConfig | HTTPValidationError
     """


    return sync_detailed(
        config_ext_id=config_ext_id,
client=client,

    ).parsed

async def asyncio_detailed(
    config_ext_id: str,
    *,
    client: AuthenticatedClient | Client,

) -> Response[AllConfigs | NonDeveloperConfig | HTTPValidationError]:
    """ Get Configs

     Read configurations from database to be displayed in the UI

    Args:
        config_ext_id (str): Config name: 'cfg-XXXXXXXX' or 'default' for system default

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AllConfigs | NonDeveloperConfig | HTTPValidationError]
     """


    kwargs = _get_kwargs(
        config_ext_id=config_ext_id,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    config_ext_id: str,
    *,
    client: AuthenticatedClient | Client,

) -> AllConfigs | NonDeveloperConfig | HTTPValidationError | None:
    """ Get Configs

     Read configurations from database to be displayed in the UI

    Args:
        config_ext_id (str): Config name: 'cfg-XXXXXXXX' or 'default' for system default

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AllConfigs | NonDeveloperConfig | HTTPValidationError
     """


    return (await asyncio_detailed(
        config_ext_id=config_ext_id,
client=client,

    )).parsed



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.update_workspace_user_roles_request import UpdateWorkspaceUserRolesRequest
from ...models.workspace_user_response import WorkspaceUserResponse
from typing import cast



def _get_kwargs(
    *,
    body: UpdateWorkspaceUserRolesRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "patch",
        "url": "/v1/workspace/users",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in (_response_200):
            response_200_item = WorkspaceUserResponse.from_dict(response_200_item_data)



            response_200.append(response_200_item)

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: UpdateWorkspaceUserRolesRequest,

) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    """ Update Workspace User Roles

     Update user roles in a workspace (bulk operation). Only workspace owners can update roles.
    Returns the full WorkspaceUserResponse for each successfully updated user.

    Args:
        body (UpdateWorkspaceUserRolesRequest): Request to update user or agent roles in a
            workspace (PATCH /workspace/{id}/users).

            All specified users/agents are updated to the same role.
            Accepts both usr-XXXXXXXX and agt-XXXXXXXX external IDs.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[WorkspaceUserResponse]]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: UpdateWorkspaceUserRolesRequest,

) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    """ Update Workspace User Roles

     Update user roles in a workspace (bulk operation). Only workspace owners can update roles.
    Returns the full WorkspaceUserResponse for each successfully updated user.

    Args:
        body (UpdateWorkspaceUserRolesRequest): Request to update user or agent roles in a
            workspace (PATCH /workspace/{id}/users).

            All specified users/agents are updated to the same role.
            Accepts both usr-XXXXXXXX and agt-XXXXXXXX external IDs.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[WorkspaceUserResponse]
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: UpdateWorkspaceUserRolesRequest,

) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    """ Update Workspace User Roles

     Update user roles in a workspace (bulk operation). Only workspace owners can update roles.
    Returns the full WorkspaceUserResponse for each successfully updated user.

    Args:
        body (UpdateWorkspaceUserRolesRequest): Request to update user or agent roles in a
            workspace (PATCH /workspace/{id}/users).

            All specified users/agents are updated to the same role.
            Accepts both usr-XXXXXXXX and agt-XXXXXXXX external IDs.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[WorkspaceUserResponse]]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: UpdateWorkspaceUserRolesRequest,

) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    """ Update Workspace User Roles

     Update user roles in a workspace (bulk operation). Only workspace owners can update roles.
    Returns the full WorkspaceUserResponse for each successfully updated user.

    Args:
        body (UpdateWorkspaceUserRolesRequest): Request to update user or agent roles in a
            workspace (PATCH /workspace/{id}/users).

            All specified users/agents are updated to the same role.
            Accepts both usr-XXXXXXXX and agt-XXXXXXXX external IDs.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[WorkspaceUserResponse]
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.add_workspace_users_request import AddWorkspaceUsersRequest
from ...models.http_validation_error import HTTPValidationError
from ...models.workspace_user_response import WorkspaceUserResponse
from typing import cast



def _get_kwargs(
    *,
    body: AddWorkspaceUsersRequest,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/workspace/users",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    if response.status_code == 201:
        response_201 = []
        _response_201 = response.json()
        for response_201_item_data in (_response_201):
            response_201_item = WorkspaceUserResponse.from_dict(response_201_item_data)



            response_201.append(response_201_item)

        return response_201

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: AddWorkspaceUsersRequest,

) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    """ Add Workspace Users

     Add users to a workspace (bulk operation). Only workspace owners can add users.

    Two modes:
    1. Body supplies ``workspace_key`` + ``workspace_ext_id``: caller provides the
       SealedBox-encrypted workspace key (encrypted with their session public key).
       The workspace need not be open/active on the session. This allows sharing any
       workspace for which the caller holds a key (e.g. from their login response).
    2. No body key: server falls back to the session's current active workspace.

    Wraps the workspace key with each recipient's X25519 public key (permanent grant)
    or deposits it into their Redis session (temporary grant when session_pubkey_b64 is set).
    Returns the full WorkspaceUserResponse for each successfully added user.

    Args:
        body (AddWorkspaceUsersRequest): Request to add users to a workspace (POST
            /workspace/users).

            All invited users receive the same role.

            workspace_key + workspace_ext_id (optional): If provided, the caller supplies the
            SealedBox-encrypted workspace key directly (encrypted with their session public key)
            and the target workspace external ID. This allows sharing any workspace the caller
            holds a key for without needing to open/activate it first. If omitted, the server
            falls back to the current session's active workspace.

            session_pubkey_b64 (optional): If set, the grant is temporary and scoped to the
            recipient's current session (identified by this X25519 public key). The workspace
            key is stored in that session's Redis entry and the workspaceusers row expires when
            the session expires. Omit for a permanent grant (default behaviour).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[WorkspaceUserResponse]]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: AddWorkspaceUsersRequest,

) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    """ Add Workspace Users

     Add users to a workspace (bulk operation). Only workspace owners can add users.

    Two modes:
    1. Body supplies ``workspace_key`` + ``workspace_ext_id``: caller provides the
       SealedBox-encrypted workspace key (encrypted with their session public key).
       The workspace need not be open/active on the session. This allows sharing any
       workspace for which the caller holds a key (e.g. from their login response).
    2. No body key: server falls back to the session's current active workspace.

    Wraps the workspace key with each recipient's X25519 public key (permanent grant)
    or deposits it into their Redis session (temporary grant when session_pubkey_b64 is set).
    Returns the full WorkspaceUserResponse for each successfully added user.

    Args:
        body (AddWorkspaceUsersRequest): Request to add users to a workspace (POST
            /workspace/users).

            All invited users receive the same role.

            workspace_key + workspace_ext_id (optional): If provided, the caller supplies the
            SealedBox-encrypted workspace key directly (encrypted with their session public key)
            and the target workspace external ID. This allows sharing any workspace the caller
            holds a key for without needing to open/activate it first. If omitted, the server
            falls back to the current session's active workspace.

            session_pubkey_b64 (optional): If set, the grant is temporary and scoped to the
            recipient's current session (identified by this X25519 public key). The workspace
            key is stored in that session's Redis entry and the workspaceusers row expires when
            the session expires. Omit for a permanent grant (default behaviour).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[WorkspaceUserResponse]
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: AddWorkspaceUsersRequest,

) -> Response[HTTPValidationError | list[WorkspaceUserResponse]]:
    """ Add Workspace Users

     Add users to a workspace (bulk operation). Only workspace owners can add users.

    Two modes:
    1. Body supplies ``workspace_key`` + ``workspace_ext_id``: caller provides the
       SealedBox-encrypted workspace key (encrypted with their session public key).
       The workspace need not be open/active on the session. This allows sharing any
       workspace for which the caller holds a key (e.g. from their login response).
    2. No body key: server falls back to the session's current active workspace.

    Wraps the workspace key with each recipient's X25519 public key (permanent grant)
    or deposits it into their Redis session (temporary grant when session_pubkey_b64 is set).
    Returns the full WorkspaceUserResponse for each successfully added user.

    Args:
        body (AddWorkspaceUsersRequest): Request to add users to a workspace (POST
            /workspace/users).

            All invited users receive the same role.

            workspace_key + workspace_ext_id (optional): If provided, the caller supplies the
            SealedBox-encrypted workspace key directly (encrypted with their session public key)
            and the target workspace external ID. This allows sharing any workspace the caller
            holds a key for without needing to open/activate it first. If omitted, the server
            falls back to the current session's active workspace.

            session_pubkey_b64 (optional): If set, the grant is temporary and scoped to the
            recipient's current session (identified by this X25519 public key). The workspace
            key is stored in that session's Redis entry and the workspaceusers row expires when
            the session expires. Omit for a permanent grant (default behaviour).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[WorkspaceUserResponse]]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: AddWorkspaceUsersRequest,

) -> HTTPValidationError | list[WorkspaceUserResponse] | None:
    """ Add Workspace Users

     Add users to a workspace (bulk operation). Only workspace owners can add users.

    Two modes:
    1. Body supplies ``workspace_key`` + ``workspace_ext_id``: caller provides the
       SealedBox-encrypted workspace key (encrypted with their session public key).
       The workspace need not be open/active on the session. This allows sharing any
       workspace for which the caller holds a key (e.g. from their login response).
    2. No body key: server falls back to the session's current active workspace.

    Wraps the workspace key with each recipient's X25519 public key (permanent grant)
    or deposits it into their Redis session (temporary grant when session_pubkey_b64 is set).
    Returns the full WorkspaceUserResponse for each successfully added user.

    Args:
        body (AddWorkspaceUsersRequest): Request to add users to a workspace (POST
            /workspace/users).

            All invited users receive the same role.

            workspace_key + workspace_ext_id (optional): If provided, the caller supplies the
            SealedBox-encrypted workspace key directly (encrypted with their session public key)
            and the target workspace external ID. This allows sharing any workspace the caller
            holds a key for without needing to open/activate it first. If omitted, the server
            falls back to the current session's active workspace.

            session_pubkey_b64 (optional): If set, the grant is temporary and scoped to the
            recipient's current session (identified by this X25519 public key). The workspace
            key is stored in that session's Redis entry and the workspaceusers row expires when
            the session expires. Omit for a permanent grant (default behaviour).

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[WorkspaceUserResponse]
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed
